@@ -25,7 +25,7 @@ def box_blur(image, radius=1, n=1):
     return image._new(image.im.box_blur(radius, n))
 
 
-def assertImage(im, data, delta=0):
+def assert_image(im, data, delta=0):
     it = iter(im.getdata())
     for data_row in data:
         im_row = [next(it) for _ in range(im.size[0])]
@@ -35,12 +35,12 @@ def assertImage(im, data, delta=0):
         next(it)
 
 
-def assertBlur(im, radius, data, passes=1, delta=0):
+def assert_blur(im, radius, data, passes=1, delta=0):
     # check grayscale image
-    assertImage(box_blur(im, radius, passes), data, delta)
+    assert_image(box_blur(im, radius, passes), data, delta)
     rgba = Image.merge("RGBA", (im, im, im, im))
     for band in box_blur(rgba, radius, passes).split():
-        assertImage(band, data, delta)
+        assert_image(band, data, delta)
 
 
 def test_color_modes():
@@ -64,7 +64,7 @@ def test_color_modes():
 
 
 def test_radius_0():
-    assertBlur(
+    assert_blur(
         sample,
         0,
         [
@@ -80,7 +80,7 @@ def test_radius_0():
 
 
 def test_radius_0_02():
-    assertBlur(
+    assert_blur(
         sample,
         0.02,
         [
@@ -97,7 +97,7 @@ def test_radius_0_02():
 
 
 def test_radius_0_05():
-    assertBlur(
+    assert_blur(
         sample,
         0.05,
         [
@@ -114,7 +114,7 @@ def test_radius_0_05():
 
 
 def test_radius_0_1():
-    assertBlur(
+    assert_blur(
         sample,
         0.1,
         [
@@ -131,7 +131,7 @@ def test_radius_0_1():
 
 
 def test_radius_0_5():
-    assertBlur(
+    assert_blur(
         sample,
         0.5,
         [
@@ -148,7 +148,7 @@ def test_radius_0_5():
 
 
 def test_radius_1():
-    assertBlur(
+    assert_blur(
         sample,
         1,
         [
@@ -165,7 +165,7 @@ def test_radius_1():
 
 
 def test_radius_1_5():
-    assertBlur(
+    assert_blur(
         sample,
         1.5,
         [
@@ -182,7 +182,7 @@ def test_radius_1_5():
 
 
 def test_radius_bigger_then_half():
-    assertBlur(
+    assert_blur(
         sample,
         3,
         [
@@ -199,7 +199,7 @@ def test_radius_bigger_then_half():
 
 
 def test_radius_bigger_then_width():
-    assertBlur(
+    assert_blur(
         sample,
         10,
         [
@@ -214,7 +214,7 @@ def test_radius_bigger_then_width():
 
 
 def test_extreme_large_radius():
-    assertBlur(
+    assert_blur(
         sample,
         600,
         [
@@ -229,7 +229,7 @@ def test_extreme_large_radius():
 
 
 def test_two_passes():
-    assertBlur(
+    assert_blur(
         sample,
         1,
         [
@@ -247,7 +247,7 @@ def test_two_passes():
 
 
 def test_three_passes():
-    assertBlur(
+    assert_blur(
         sample,
         1,
         [
