@@ -10,14 +10,20 @@ from .helper import assert_image_equal, assert_image_equal_tofile, hopper
 TEST_FILE_DXT1 = "Tests/images/dxt1-rgb-4bbp-noalpha_MipMaps-1.dds"
 TEST_FILE_DXT3 = "Tests/images/dxt3-argb-8bbp-explicitalpha_MipMaps-1.dds"
 TEST_FILE_DXT5 = "Tests/images/dxt5-argb-8bbp-interpolatedalpha_MipMaps-1.dds"
+TEST_FILE_ATI1 = "Tests/images/ati1.dds"
+TEST_FILE_ATI2 = "Tests/images/ati2.dds"
 TEST_FILE_DX10_BC5_TYPELESS = "Tests/images/bc5_typeless.dds"
 TEST_FILE_DX10_BC5_UNORM = "Tests/images/bc5_unorm.dds"
 TEST_FILE_DX10_BC5_SNORM = "Tests/images/bc5_snorm.dds"
 TEST_FILE_BC5S = "Tests/images/bc5s.dds"
+TEST_FILE_BC6H = "Tests/images/bc6h.dds"
+TEST_FILE_BC6HS = "Tests/images/bc6h_sf.dds"
 TEST_FILE_DX10_BC7 = "Tests/images/bc7-argb-8bpp_MipMaps-1.dds"
 TEST_FILE_DX10_BC7_UNORM_SRGB = "Tests/images/DXGI_FORMAT_BC7_UNORM_SRGB.dds"
 TEST_FILE_DX10_R8G8B8A8 = "Tests/images/argb-32bpp_MipMaps-1.dds"
 TEST_FILE_DX10_R8G8B8A8_UNORM_SRGB = "Tests/images/DXGI_FORMAT_R8G8B8A8_UNORM_SRGB.dds"
+TEST_FILE_UNCOMPRESSED_L = "Tests/images/uncompressed_l.dds"
+TEST_FILE_UNCOMPRESSED_L_WITH_ALPHA = "Tests/images/uncompressed_la.dds"
 TEST_FILE_UNCOMPRESSED_RGB = "Tests/images/hopper.dds"
 TEST_FILE_UNCOMPRESSED_RGB_WITH_ALPHA = "Tests/images/uncompressed_rgb.dds"
 
@@ -62,6 +68,32 @@ def test_sanity_dxt5():
     assert_image_equal_tofile(im, TEST_FILE_DXT5.replace(".dds", ".png"))
 
 
+def test_sanity_ati1():
+    """Check ATI1 images can be opened"""
+
+    with Image.open(TEST_FILE_ATI1) as im:
+        im.load()
+
+        assert im.format == "DDS"
+        assert im.mode == "L"
+        assert im.size == (64, 64)
+
+        assert_image_equal_tofile(im, TEST_FILE_ATI1.replace(".dds", ".png"))
+
+
+def test_sanity_ati2():
+    """Check ATI2 images can be opened"""
+
+    with Image.open(TEST_FILE_ATI2) as im:
+        im.load()
+
+        assert im.format == "DDS"
+        assert im.mode == "RGB"
+        assert im.size == (256, 256)
+
+        assert_image_equal_tofile(im, TEST_FILE_DX10_BC5_UNORM.replace(".dds", ".png"))
+
+
 @pytest.mark.parametrize(
     ("image_path", "expected_path"),
     (
@@ -86,6 +118,20 @@ def test_dx10_bc5(image_path, expected_path):
         assert_image_equal_tofile(im, expected_path.replace(".dds", ".png"))
 
 
+@pytest.mark.parametrize("image_path", (TEST_FILE_BC6H, TEST_FILE_BC6HS))
+def test_dx10_bc6h(image_path):
+    """Check DX10 BC6H/BC6HS images can be opened"""
+
+    with Image.open(image_path) as im:
+        im.load()
+
+        assert im.format == "DDS"
+        assert im.mode == "RGB"
+        assert im.size == (128, 128)
+
+        assert_image_equal_tofile(im, image_path.replace(".dds", ".png"))
+
+
 def test_dx10_bc7():
     """Check DX10 images can be opened"""
 
@@ -150,26 +196,24 @@ def test_unimplemented_dxgi_format():
             pass
 
 
-def test_uncompressed_rgb():
-    """Check uncompressed RGB images can be opened"""
+@pytest.mark.parametrize(
+    ("mode", "size", "test_file"),
+    [
+        ("L", (128, 128), TEST_FILE_UNCOMPRESSED_L),
+        ("LA", (128, 128), TEST_FILE_UNCOMPRESSED_L_WITH_ALPHA),
+        ("RGB", (128, 128), TEST_FILE_UNCOMPRESSED_RGB),
+        ("RGBA", (800, 600), TEST_FILE_UNCOMPRESSED_RGB_WITH_ALPHA),
+    ],
+)
+def test_uncompressed(mode, size, test_file):
+    """Check uncompressed images can be opened"""
 
-    # convert -format dds -define dds:compression=none hopper.jpg hopper.dds
-    with Image.open(TEST_FILE_UNCOMPRESSED_RGB) as im:
+    with Image.open(test_file) as im:
         assert im.format == "DDS"
-        assert im.mode == "RGB"
-        assert im.size == (128, 128)
+        assert im.mode == mode
+        assert im.size == size
 
-        assert_image_equal_tofile(im, "Tests/images/hopper.png")
-
-    # Test image with alpha
-    with Image.open(TEST_FILE_UNCOMPRESSED_RGB_WITH_ALPHA) as im:
-        assert im.format == "DDS"
-        assert im.mode == "RGBA"
-        assert im.size == (800, 600)
-
-        assert_image_equal_tofile(
-            im, TEST_FILE_UNCOMPRESSED_RGB_WITH_ALPHA.replace(".dds", ".png")
-        )
+        assert_image_equal_tofile(im, test_file.replace(".dds", ".png"))
 
 
 def test__accept_true():
@@ -196,6 +240,13 @@ def test__accept_false():
     assert not output
 
 
+def test_invalid_file():
+    invalid_file = "Tests/images/flower.jpg"
+
+    with pytest.raises(SyntaxError):
+        DdsImagePlugin.DdsImageFile(invalid_file)
+
+
 def test_short_header():
     """Check a short header"""
     with open(TEST_FILE_DXT5, "rb") as f:
@@ -254,6 +305,8 @@ def test_save_unsupported_mode(tmp_path):
 @pytest.mark.parametrize(
     ("mode", "test_file"),
     [
+        ("L", "Tests/images/linear_gradient.png"),
+        ("LA", "Tests/images/uncompressed_la.png"),
         ("RGB", "Tests/images/hopper.png"),
         ("RGBA", "Tests/images/pil123rgba.png"),
     ],
