@@ -83,18 +83,40 @@ def test_l_mode_transparency():
         assert im.load()[0, 0] == 128
 
 
+def test_l_mode_after_rgb():
+    with Image.open("Tests/images/no_palette_after_rgb.gif") as im:
+        im.seek(1)
+        assert im.mode == "RGB"
+
+        im.seek(2)
+        assert im.mode == "RGB"
+
+
+def test_palette_not_needed_for_second_frame():
+    with Image.open("Tests/images/palette_not_needed_for_second_frame.gif") as im:
+        im.seek(1)
+        assert_image_similar(im, hopper("L").convert("RGB"), 8)
+
+
 def test_strategy():
+    with Image.open("Tests/images/iss634.gif") as im:
+        expected_rgb_always = im.convert("RGB")
+
     with Image.open("Tests/images/chi.gif") as im:
-        expected_zero = im.convert("RGB")
+        expected_rgb_always_rgba = im.convert("RGBA")
 
         im.seek(1)
-        expected_one = im.convert("RGB")
+        expected_different = im.convert("RGB")
 
     try:
         GifImagePlugin.LOADING_STRATEGY = GifImagePlugin.LoadingStrategy.RGB_ALWAYS
-        with Image.open("Tests/images/chi.gif") as im:
+        with Image.open("Tests/images/iss634.gif") as im:
             assert im.mode == "RGB"
-            assert_image_equal(im, expected_zero)
+            assert_image_equal(im, expected_rgb_always)
+
+        with Image.open("Tests/images/chi.gif") as im:
+            assert im.mode == "RGBA"
+            assert_image_equal(im, expected_rgb_always_rgba)
 
         GifImagePlugin.LOADING_STRATEGY = (
             GifImagePlugin.LoadingStrategy.RGB_AFTER_DIFFERENT_PALETTE_ONLY
@@ -105,7 +127,7 @@ def test_strategy():
 
             im.seek(1)
             assert im.mode == "P"
-            assert_image_equal(im.convert("RGB"), expected_one)
+            assert_image_equal(im.convert("RGB"), expected_different)
 
         # Change to RGB mode when a frame has an individual palette
         with Image.open("Tests/images/iss634.gif") as im:
@@ -399,6 +421,11 @@ def test_no_change():
         assert im.is_animated
         assert_image_equal(im, expected)
 
+    with Image.open("Tests/images/comment_after_only_frame.gif") as im:
+        expected = Image.new("P", (1, 1))
+        assert not im.is_animated
+        assert_image_equal(im, expected)
+
 
 def test_eoferror():
     with Image.open(TEST_GIF) as im:
@@ -650,6 +677,24 @@ def test_dispose2_background(tmp_path):
         assert im.getpixel((0, 0)) == (255, 0, 0)
 
 
+def test_dispose2_background_frame(tmp_path):
+    out = str(tmp_path / "temp.gif")
+
+    im_list = [Image.new("RGBA", (1, 20))]
+
+    different_frame = Image.new("RGBA", (1, 20))
+    different_frame.putpixel((0, 10), (255, 0, 0, 255))
+    im_list.append(different_frame)
+
+    # Frame that matches the background
+    im_list.append(Image.new("RGBA", (1, 20)))
+
+    im_list[0].save(out, save_all=True, append_images=im_list[1:], disposal=2)
+
+    with Image.open(out) as im:
+        assert im.n_frames == 3
+
+
 def test_transparency_in_second_frame(tmp_path):
     out = str(tmp_path / "temp.gif")
     with Image.open("Tests/images/different_transparency.gif") as im:
@@ -764,6 +809,22 @@ def test_roundtrip_info_duration(tmp_path):
         ] == duration_list
 
 
+def test_roundtrip_info_duration_combined(tmp_path):
+    out = str(tmp_path / "temp.gif")
+    with Image.open("Tests/images/duplicate_frame.gif") as im:
+        assert [frame.info["duration"] for frame in ImageSequence.Iterator(im)] == [
+            1000,
+            1000,
+            1000,
+        ]
+        im.save(out, save_all=True)
+
+    with Image.open(out) as reloaded:
+        assert [
+            frame.info["duration"] for frame in ImageSequence.Iterator(reloaded)
+        ] == [1000, 2000]
+
+
 def test_identical_frames(tmp_path):
     duration_list = [1000, 1500, 2000, 4000]
 
@@ -788,24 +849,24 @@ def test_identical_frames(tmp_path):
         assert reread.info["duration"] == 4500
 
 
-def test_identical_frames_to_single_frame(tmp_path):
-    for duration in ([1000, 1500, 2000, 4000], (1000, 1500, 2000, 4000), 8500):
-        out = str(tmp_path / "temp.gif")
-        im_list = [
-            Image.new("L", (100, 100), "#000"),
-            Image.new("L", (100, 100), "#000"),
-            Image.new("L", (100, 100), "#000"),
-        ]
+@pytest.mark.parametrize(
+    "duration", ([1000, 1500, 2000, 4000], (1000, 1500, 2000, 4000), 8500)
+)
+def test_identical_frames_to_single_frame(duration, tmp_path):
+    out = str(tmp_path / "temp.gif")
+    im_list = [
+        Image.new("L", (100, 100), "#000"),
+        Image.new("L", (100, 100), "#000"),
+        Image.new("L", (100, 100), "#000"),
+    ]
 
-        im_list[0].save(
-            out, save_all=True, append_images=im_list[1:], duration=duration
-        )
-        with Image.open(out) as reread:
-            # Assert that all frames were combined
-            assert reread.n_frames == 1
+    im_list[0].save(out, save_all=True, append_images=im_list[1:], duration=duration)
+    with Image.open(out) as reread:
+        # Assert that all frames were combined
+        assert reread.n_frames == 1
 
-            # Assert that the new duration is the total of the identical frames
-            assert reread.info["duration"] == 8500
+        # Assert that the new duration is the total of the identical frames
+        assert reread.info["duration"] == 8500
 
 
 def test_number_of_loops(tmp_path):
@@ -832,14 +893,23 @@ def test_background(tmp_path):
     im.info["background"] = 1
     im.save(out)
     with Image.open(out) as reread:
-
         assert reread.info["background"] == im.info["background"]
 
+
+def test_webp_background(tmp_path):
+    out = str(tmp_path / "temp.gif")
+
+    # Test opaque WebP background
     if features.check("webp") and features.check("webp_anim"):
         with Image.open("Tests/images/hopper.webp") as im:
-            assert isinstance(im.info["background"], tuple)
+            assert im.info["background"] == (255, 255, 255, 255)
             im.save(out)
 
+    # Test non-opaque WebP background
+    im = Image.new("L", (100, 100), "#000")
+    im.info["background"] = (0, 0, 0, 0)
+    im.save(out)
+
 
 def test_comment(tmp_path):
     with Image.open(TEST_GIF) as im:
@@ -1082,6 +1152,19 @@ def test_palette_save_P(tmp_path):
         assert_image_equal(reloaded, im)
 
 
+def test_palette_save_duplicate_entries(tmp_path):
+    im = Image.new("P", (1, 2))
+    im.putpixel((0, 1), 1)
+
+    im.putpalette((0, 0, 0, 0, 0, 0))
+
+    out = str(tmp_path / "temp.gif")
+    im.save(out, palette=[0, 0, 0, 0, 0, 0, 1, 1, 1])
+
+    with Image.open(out) as reloaded:
+        assert reloaded.convert("RGB").getpixel((0, 1)) == (0, 0, 0)
+
+
 def test_palette_save_all_P(tmp_path):
     frames = []
     colors = ((255, 0, 0), (0, 255, 0))
