@@ -1,4 +1,5 @@
 import tempfile
+import warnings
 from io import BytesIO
 
 import pytest
@@ -28,21 +29,17 @@ def test_unclosed_file():
 
 
 def test_closed_file():
-    with pytest.warns(None) as record:
+    with warnings.catch_warnings():
         im = Image.open(TEST_FILE)
         im.load()
         im.close()
 
-    assert not record
-
 
 def test_context_manager():
-    with pytest.warns(None) as record:
+    with warnings.catch_warnings():
         with Image.open(TEST_FILE) as im:
             im.load()
 
-    assert not record
-
 
 def test_save(tmp_path):
     # Arrange
