@@ -2,15 +2,11 @@ from PIL import WalImageFile
 
 from .helper import assert_image_equal_tofile
 
+TEST_FILE = "Tests/images/hopper.wal"
+
 
 def test_open():
-    # Arrange
-    TEST_FILE = "Tests/images/hopper.wal"
-
-    # Act
     with WalImageFile.open(TEST_FILE) as im:
-
-        # Assert
         assert im.format == "WAL"
         assert im.format_description == "Quake2 Texture"
         assert im.mode == "P"
@@ -19,3 +15,11 @@ def test_open():
         assert isinstance(im, WalImageFile.WalImageFile)
 
         assert_image_equal_tofile(im, "Tests/images/hopper_wal.png")
+
+
+def test_load():
+    with WalImageFile.open(TEST_FILE) as im:
+        assert im.load()[0, 0] == 122
+
+        # Test again now that it has already been loaded once
+        assert im.load()[0, 0] == 122
