@@ -11,6 +11,11 @@ pytestmark = [
     skip_unless_feature("webp_mux"),
 ]
 
+try:
+    from defusedxml import ElementTree
+except ImportError:
+    ElementTree = None
+
 
 def test_read_exif_metadata():
 
@@ -55,9 +60,7 @@ def test_write_exif_metadata():
     test_buffer.seek(0)
     with Image.open(test_buffer) as webp_image:
         webp_exif = webp_image.info.get("exif", None)
-    assert webp_exif
-    if webp_exif:
-        assert webp_exif == expected_exif, "WebP EXIF didn't match"
+    assert webp_exif == expected_exif[6:], "WebP EXIF didn't match"
 
 
 def test_read_icc_profile():
@@ -112,6 +115,22 @@ def test_read_no_exif():
         assert not webp_image._getexif()
 
 
+def test_getxmp():
+    with Image.open("Tests/images/flower.webp") as im:
+        assert "xmp" not in im.info
+        assert im.getxmp() == {}
+
+    with Image.open("Tests/images/flower2.webp") as im:
+        if ElementTree is None:
+            with pytest.warns(UserWarning):
+                assert im.getxmp() == {}
+        else:
+            assert (
+                im.getxmp()["xmpmeta"]["xmptk"]
+                == "Adobe XMP Core 5.3-c011 66.145661, 2012/02/06-14:56:27        "
+            )
+
+
 @skip_unless_feature("webp_anim")
 def test_write_animated_metadata(tmp_path):
     iccp_data = b"<iccp_data>"
