@@ -1,8 +1,9 @@
 import pytest
+from packaging.version import parse as parse_version
 
-from PIL import Image
+from PIL import Image, features
 
-from .helper import assert_image_similar, hopper, is_ppc64le
+from .helper import assert_image_similar, hopper, is_ppc64le, skip_unless_feature
 
 
 def test_sanity():
@@ -17,16 +18,14 @@ def test_sanity():
     assert_image_similar(converted.convert("RGB"), image, 60)
 
 
-@pytest.mark.xfail(is_ppc64le(), reason="failing on ppc64le on GHA")
+@skip_unless_feature("libimagequant")
 def test_libimagequant_quantize():
     image = hopper()
-    try:
-        converted = image.quantize(100, Image.LIBIMAGEQUANT)
-    except ValueError as ex:  # pragma: no cover
-        if "dependency" in str(ex).lower():
-            pytest.skip("libimagequant support not available")
-        else:
-            raise
+    if is_ppc64le():
+        libimagequant = parse_version(features.version_feature("libimagequant"))
+        if libimagequant < parse_version("4"):
+            pytest.skip("Fails with libimagequant earlier than 4.0.0 on ppc64le")
+    converted = image.quantize(100, Image.Quantize.LIBIMAGEQUANT)
     assert converted.mode == "P"
     assert_image_similar(converted.convert("RGB"), image, 15)
     assert len(converted.getcolors()) == 100
@@ -34,7 +33,7 @@ def test_libimagequant_quantize():
 
 def test_octree_quantize():
     image = hopper()
-    converted = image.quantize(100, Image.FASTOCTREE)
+    converted = image.quantize(100, Image.Quantize.FASTOCTREE)
     assert converted.mode == "P"
     assert_image_similar(converted.convert("RGB"), image, 20)
     assert len(converted.getcolors()) == 100
@@ -61,18 +60,34 @@ def test_quantize_no_dither():
     with Image.open("Tests/images/caption_6_33_22.png") as palette:
         palette = palette.convert("P")
 
-    converted = image.quantize(dither=0, palette=palette)
+    converted = image.quantize(dither=Image.Dither.NONE, palette=palette)
     assert converted.mode == "P"
     assert converted.palette.palette == palette.palette.palette
 
 
+def test_quantize_no_dither2():
+    im = Image.new("RGB", (9, 1))
+    im.putdata(list((p,) * 3 for p in range(0, 36, 4)))
+
+    palette = Image.new("P", (1, 1))
+    data = (0, 0, 0, 32, 32, 32)
+    palette.putpalette(data)
+    quantized = im.quantize(dither=Image.Dither.NONE, palette=palette)
+
+    assert tuple(quantized.palette.palette) == data
+
+    px = quantized.load()
+    for x in range(9):
+        assert px[x, 0] == (0 if x < 5 else 1)
+
+
 def test_quantize_dither_diff():
     image = hopper()
     with Image.open("Tests/images/caption_6_33_22.png") as palette:
         palette = palette.convert("P")
 
-    dither = image.quantize(dither=1, palette=palette)
-    nodither = image.quantize(dither=0, palette=palette)
+    dither = image.quantize(dither=Image.Dither.FLOYDSTEINBERG, palette=palette)
+    nodither = image.quantize(dither=Image.Dither.NONE, palette=palette)
 
     assert dither.tobytes() != nodither.tobytes()
 
@@ -97,10 +112,10 @@ def test_transparent_colors_equal():
 @pytest.mark.parametrize(
     "method, color",
     (
-        (Image.MEDIANCUT, (0, 0, 0)),
-        (Image.MAXCOVERAGE, (0, 0, 0)),
-        (Image.FASTOCTREE, (0, 0, 0)),
-        (Image.FASTOCTREE, (0, 0, 0, 0)),
+        (Image.Quantize.MEDIANCUT, (0, 0, 0)),
+        (Image.Quantize.MAXCOVERAGE, (0, 0, 0)),
+        (Image.Quantize.FASTOCTREE, (0, 0, 0)),
+        (Image.Quantize.FASTOCTREE, (0, 0, 0, 0)),
     ),
 )
 def test_palette(method, color):
@@ -109,3 +124,18 @@ def test_palette(method, color):
     converted = im.quantize(method=method)
     converted_px = converted.load()
     assert converted_px[0, 0] == converted.palette.colors[color]
+
+
+def test_small_palette():
+    # Arrange
+    im = hopper()
+
+    colors = (255, 0, 0, 0, 0, 255)
+    p = Image.new("P", (1, 1))
+    p.putpalette(colors)
+
+    # Act
+    im = im.quantize(palette=p)
+
+    # Assert
+    assert len(im.getcolors()) == 2
