@@ -24,7 +24,7 @@ class TestImagingResampleVulnerability:
         ):
             with pytest.raises(MemoryError):
                 # any resampling filter will do here
-                im.im.resize((xsize, ysize), Image.BILINEAR)
+                im.im.resize((xsize, ysize), Image.Resampling.BILINEAR)
 
     def test_invalid_size(self):
         im = hopper()
@@ -100,43 +100,44 @@ class TestImagingCoreResampleAccuracy:
             for y in range(image.size[1])
         )
 
-    def test_reduce_box(self):
-        for mode in ["RGBX", "RGB", "La", "L"]:
-            case = self.make_case(mode, (8, 8), 0xE1)
-            case = case.resize((4, 4), Image.BOX)
-            # fmt: off
-            data = ("e1 e1"
-                    "e1 e1")
-            # fmt: on
-            for channel in case.split():
-                self.check_case(channel, self.make_sample(data, (4, 4)))
+    @pytest.mark.parametrize("mode", ("RGBX", "RGB", "La", "L"))
+    def test_reduce_box(self, mode):
+        case = self.make_case(mode, (8, 8), 0xE1)
+        case = case.resize((4, 4), Image.Resampling.BOX)
+        # fmt: off
+        data = ("e1 e1"
+                "e1 e1")
+        # fmt: on
+        for channel in case.split():
+            self.check_case(channel, self.make_sample(data, (4, 4)))
 
-    def test_reduce_bilinear(self):
-        for mode in ["RGBX", "RGB", "La", "L"]:
-            case = self.make_case(mode, (8, 8), 0xE1)
-            case = case.resize((4, 4), Image.BILINEAR)
-            # fmt: off
-            data = ("e1 c9"
-                    "c9 b7")
-            # fmt: on
-            for channel in case.split():
-                self.check_case(channel, self.make_sample(data, (4, 4)))
+    @pytest.mark.parametrize("mode", ("RGBX", "RGB", "La", "L"))
+    def test_reduce_bilinear(self, mode):
+        case = self.make_case(mode, (8, 8), 0xE1)
+        case = case.resize((4, 4), Image.Resampling.BILINEAR)
+        # fmt: off
+        data = ("e1 c9"
+                "c9 b7")
+        # fmt: on
+        for channel in case.split():
+            self.check_case(channel, self.make_sample(data, (4, 4)))
 
-    def test_reduce_hamming(self):
-        for mode in ["RGBX", "RGB", "La", "L"]:
-            case = self.make_case(mode, (8, 8), 0xE1)
-            case = case.resize((4, 4), Image.HAMMING)
-            # fmt: off
-            data = ("e1 da"
-                    "da d3")
-            # fmt: on
-            for channel in case.split():
-                self.check_case(channel, self.make_sample(data, (4, 4)))
+    @pytest.mark.parametrize("mode", ("RGBX", "RGB", "La", "L"))
+    def test_reduce_hamming(self, mode):
+        case = self.make_case(mode, (8, 8), 0xE1)
+        case = case.resize((4, 4), Image.Resampling.HAMMING)
+        # fmt: off
+        data = ("e1 da"
+                "da d3")
+        # fmt: on
+        for channel in case.split():
+            self.check_case(channel, self.make_sample(data, (4, 4)))
 
-    def test_reduce_bicubic(self):
+    @pytest.mark.parametrize("mode", ("RGBX", "RGB", "La", "L"))
+    def test_reduce_bicubic(self, mode):
         for mode in ["RGBX", "RGB", "La", "L"]:
             case = self.make_case(mode, (12, 12), 0xE1)
-            case = case.resize((6, 6), Image.BICUBIC)
+            case = case.resize((6, 6), Image.Resampling.BICUBIC)
             # fmt: off
             data = ("e1 e3 d4"
                     "e3 e5 d6"
@@ -145,82 +146,84 @@ class TestImagingCoreResampleAccuracy:
             for channel in case.split():
                 self.check_case(channel, self.make_sample(data, (6, 6)))
 
-    def test_reduce_lanczos(self):
-        for mode in ["RGBX", "RGB", "La", "L"]:
-            case = self.make_case(mode, (16, 16), 0xE1)
-            case = case.resize((8, 8), Image.LANCZOS)
-            # fmt: off
-            data = ("e1 e0 e4 d7"
-                    "e0 df e3 d6"
-                    "e4 e3 e7 da"
-                    "d7 d6 d9 ce")
-            # fmt: on
-            for channel in case.split():
-                self.check_case(channel, self.make_sample(data, (8, 8)))
+    @pytest.mark.parametrize("mode", ("RGBX", "RGB", "La", "L"))
+    def test_reduce_lanczos(self, mode):
+        case = self.make_case(mode, (16, 16), 0xE1)
+        case = case.resize((8, 8), Image.Resampling.LANCZOS)
+        # fmt: off
+        data = ("e1 e0 e4 d7"
+                "e0 df e3 d6"
+                "e4 e3 e7 da"
+                "d7 d6 d9 ce")
+        # fmt: on
+        for channel in case.split():
+            self.check_case(channel, self.make_sample(data, (8, 8)))
 
-    def test_enlarge_box(self):
-        for mode in ["RGBX", "RGB", "La", "L"]:
-            case = self.make_case(mode, (2, 2), 0xE1)
-            case = case.resize((4, 4), Image.BOX)
-            # fmt: off
-            data = ("e1 e1"
-                    "e1 e1")
-            # fmt: on
-            for channel in case.split():
-                self.check_case(channel, self.make_sample(data, (4, 4)))
+    @pytest.mark.parametrize("mode", ("RGBX", "RGB", "La", "L"))
+    def test_enlarge_box(self, mode):
+        case = self.make_case(mode, (2, 2), 0xE1)
+        case = case.resize((4, 4), Image.Resampling.BOX)
+        # fmt: off
+        data = ("e1 e1"
+                "e1 e1")
+        # fmt: on
+        for channel in case.split():
+            self.check_case(channel, self.make_sample(data, (4, 4)))
 
-    def test_enlarge_bilinear(self):
-        for mode in ["RGBX", "RGB", "La", "L"]:
-            case = self.make_case(mode, (2, 2), 0xE1)
-            case = case.resize((4, 4), Image.BILINEAR)
-            # fmt: off
-            data = ("e1 b0"
-                    "b0 98")
-            # fmt: on
-            for channel in case.split():
-                self.check_case(channel, self.make_sample(data, (4, 4)))
+    @pytest.mark.parametrize("mode", ("RGBX", "RGB", "La", "L"))
+    def test_enlarge_bilinear(self, mode):
+        case = self.make_case(mode, (2, 2), 0xE1)
+        case = case.resize((4, 4), Image.Resampling.BILINEAR)
+        # fmt: off
+        data = ("e1 b0"
+                "b0 98")
+        # fmt: on
+        for channel in case.split():
+            self.check_case(channel, self.make_sample(data, (4, 4)))
 
-    def test_enlarge_hamming(self):
-        for mode in ["RGBX", "RGB", "La", "L"]:
-            case = self.make_case(mode, (2, 2), 0xE1)
-            case = case.resize((4, 4), Image.HAMMING)
-            # fmt: off
-            data = ("e1 d2"
-                    "d2 c5")
-            # fmt: on
-            for channel in case.split():
-                self.check_case(channel, self.make_sample(data, (4, 4)))
+    @pytest.mark.parametrize("mode", ("RGBX", "RGB", "La", "L"))
+    def test_enlarge_hamming(self, mode):
+        case = self.make_case(mode, (2, 2), 0xE1)
+        case = case.resize((4, 4), Image.Resampling.HAMMING)
+        # fmt: off
+        data = ("e1 d2"
+                "d2 c5")
+        # fmt: on
+        for channel in case.split():
+            self.check_case(channel, self.make_sample(data, (4, 4)))
 
-    def test_enlarge_bicubic(self):
-        for mode in ["RGBX", "RGB", "La", "L"]:
-            case = self.make_case(mode, (4, 4), 0xE1)
-            case = case.resize((8, 8), Image.BICUBIC)
-            # fmt: off
-            data = ("e1 e5 ee b9"
-                    "e5 e9 f3 bc"
-                    "ee f3 fd c1"
-                    "b9 bc c1 a2")
-            # fmt: on
-            for channel in case.split():
-                self.check_case(channel, self.make_sample(data, (8, 8)))
+    @pytest.mark.parametrize("mode", ("RGBX", "RGB", "La", "L"))
+    def test_enlarge_bicubic(self, mode):
+        case = self.make_case(mode, (4, 4), 0xE1)
+        case = case.resize((8, 8), Image.Resampling.BICUBIC)
+        # fmt: off
+        data = ("e1 e5 ee b9"
+                "e5 e9 f3 bc"
+                "ee f3 fd c1"
+                "b9 bc c1 a2")
+        # fmt: on
+        for channel in case.split():
+            self.check_case(channel, self.make_sample(data, (8, 8)))
 
-    def test_enlarge_lanczos(self):
-        for mode in ["RGBX", "RGB", "La", "L"]:
-            case = self.make_case(mode, (6, 6), 0xE1)
-            case = case.resize((12, 12), Image.LANCZOS)
-            data = (
-                "e1 e0 db ed f5 b8"
-                "e0 df da ec f3 b7"
-                "db db d6 e7 ee b5"
-                "ed ec e6 fb ff bf"
-                "f5 f4 ee ff ff c4"
-                "b8 b7 b4 bf c4 a0"
-            )
-            for channel in case.split():
-                self.check_case(channel, self.make_sample(data, (12, 12)))
+    @pytest.mark.parametrize("mode", ("RGBX", "RGB", "La", "L"))
+    def test_enlarge_lanczos(self, mode):
+        case = self.make_case(mode, (6, 6), 0xE1)
+        case = case.resize((12, 12), Image.Resampling.LANCZOS)
+        data = (
+            "e1 e0 db ed f5 b8"
+            "e0 df da ec f3 b7"
+            "db db d6 e7 ee b5"
+            "ed ec e6 fb ff bf"
+            "f5 f4 ee ff ff c4"
+            "b8 b7 b4 bf c4 a0"
+        )
+        for channel in case.split():
+            self.check_case(channel, self.make_sample(data, (12, 12)))
 
     def test_box_filter_correct_range(self):
-        im = Image.new("RGB", (8, 8), "#1688ff").resize((100, 100), Image.BOX)
+        im = Image.new("RGB", (8, 8), "#1688ff").resize(
+            (100, 100), Image.Resampling.BOX
+        )
         ref = Image.new("RGB", (100, 100), "#1688ff")
         assert_image_equal(im, ref)
 
@@ -228,7 +231,7 @@ class TestImagingCoreResampleAccuracy:
 class TestCoreResampleConsistency:
     def make_case(self, mode, fill):
         im = Image.new(mode, (512, 9), fill)
-        return im.resize((9, 512), Image.LANCZOS), im.load()[0, 0]
+        return im.resize((9, 512), Image.Resampling.LANCZOS), im.load()[0, 0]
 
     def run_case(self, case):
         channel, color = case
@@ -283,20 +286,20 @@ class TestCoreResampleAlphaCorrect:
     @pytest.mark.xfail(reason="Current implementation isn't precise enough")
     def test_levels_rgba(self):
         case = self.make_levels_case("RGBA")
-        self.run_levels_case(case.resize((512, 32), Image.BOX))
-        self.run_levels_case(case.resize((512, 32), Image.BILINEAR))
-        self.run_levels_case(case.resize((512, 32), Image.HAMMING))
-        self.run_levels_case(case.resize((512, 32), Image.BICUBIC))
-        self.run_levels_case(case.resize((512, 32), Image.LANCZOS))
+        self.run_levels_case(case.resize((512, 32), Image.Resampling.BOX))
+        self.run_levels_case(case.resize((512, 32), Image.Resampling.BILINEAR))
+        self.run_levels_case(case.resize((512, 32), Image.Resampling.HAMMING))
+        self.run_levels_case(case.resize((512, 32), Image.Resampling.BICUBIC))
+        self.run_levels_case(case.resize((512, 32), Image.Resampling.LANCZOS))
 
     @pytest.mark.xfail(reason="Current implementation isn't precise enough")
     def test_levels_la(self):
         case = self.make_levels_case("LA")
-        self.run_levels_case(case.resize((512, 32), Image.BOX))
-        self.run_levels_case(case.resize((512, 32), Image.BILINEAR))
-        self.run_levels_case(case.resize((512, 32), Image.HAMMING))
-        self.run_levels_case(case.resize((512, 32), Image.BICUBIC))
-        self.run_levels_case(case.resize((512, 32), Image.LANCZOS))
+        self.run_levels_case(case.resize((512, 32), Image.Resampling.BOX))
+        self.run_levels_case(case.resize((512, 32), Image.Resampling.BILINEAR))
+        self.run_levels_case(case.resize((512, 32), Image.Resampling.HAMMING))
+        self.run_levels_case(case.resize((512, 32), Image.Resampling.BICUBIC))
+        self.run_levels_case(case.resize((512, 32), Image.Resampling.LANCZOS))
 
     def make_dirty_case(self, mode, clean_pixel, dirty_pixel):
         i = Image.new(mode, (64, 64), dirty_pixel)
@@ -321,19 +324,27 @@ class TestCoreResampleAlphaCorrect:
 
     def test_dirty_pixels_rgba(self):
         case = self.make_dirty_case("RGBA", (255, 255, 0, 128), (0, 0, 255, 0))
-        self.run_dirty_case(case.resize((20, 20), Image.BOX), (255, 255, 0))
-        self.run_dirty_case(case.resize((20, 20), Image.BILINEAR), (255, 255, 0))
-        self.run_dirty_case(case.resize((20, 20), Image.HAMMING), (255, 255, 0))
-        self.run_dirty_case(case.resize((20, 20), Image.BICUBIC), (255, 255, 0))
-        self.run_dirty_case(case.resize((20, 20), Image.LANCZOS), (255, 255, 0))
+        self.run_dirty_case(case.resize((20, 20), Image.Resampling.BOX), (255, 255, 0))
+        self.run_dirty_case(
+            case.resize((20, 20), Image.Resampling.BILINEAR), (255, 255, 0)
+        )
+        self.run_dirty_case(
+            case.resize((20, 20), Image.Resampling.HAMMING), (255, 255, 0)
+        )
+        self.run_dirty_case(
+            case.resize((20, 20), Image.Resampling.BICUBIC), (255, 255, 0)
+        )
+        self.run_dirty_case(
+            case.resize((20, 20), Image.Resampling.LANCZOS), (255, 255, 0)
+        )
 
     def test_dirty_pixels_la(self):
         case = self.make_dirty_case("LA", (255, 128), (0, 0))
-        self.run_dirty_case(case.resize((20, 20), Image.BOX), (255,))
-        self.run_dirty_case(case.resize((20, 20), Image.BILINEAR), (255,))
-        self.run_dirty_case(case.resize((20, 20), Image.HAMMING), (255,))
-        self.run_dirty_case(case.resize((20, 20), Image.BICUBIC), (255,))
-        self.run_dirty_case(case.resize((20, 20), Image.LANCZOS), (255,))
+        self.run_dirty_case(case.resize((20, 20), Image.Resampling.BOX), (255,))
+        self.run_dirty_case(case.resize((20, 20), Image.Resampling.BILINEAR), (255,))
+        self.run_dirty_case(case.resize((20, 20), Image.Resampling.HAMMING), (255,))
+        self.run_dirty_case(case.resize((20, 20), Image.Resampling.BICUBIC), (255,))
+        self.run_dirty_case(case.resize((20, 20), Image.Resampling.LANCZOS), (255,))
 
 
 class TestCoreResamplePasses:
@@ -346,26 +357,26 @@ class TestCoreResamplePasses:
     def test_horizontal(self):
         im = hopper("L")
         with self.count(1):
-            im.resize((im.size[0] - 10, im.size[1]), Image.BILINEAR)
+            im.resize((im.size[0] - 10, im.size[1]), Image.Resampling.BILINEAR)
 
     def test_vertical(self):
         im = hopper("L")
         with self.count(1):
-            im.resize((im.size[0], im.size[1] - 10), Image.BILINEAR)
+            im.resize((im.size[0], im.size[1] - 10), Image.Resampling.BILINEAR)
 
     def test_both(self):
         im = hopper("L")
         with self.count(2):
-            im.resize((im.size[0] - 10, im.size[1] - 10), Image.BILINEAR)
+            im.resize((im.size[0] - 10, im.size[1] - 10), Image.Resampling.BILINEAR)
 
     def test_box_horizontal(self):
         im = hopper("L")
         box = (20, 0, im.size[0] - 20, im.size[1])
         with self.count(1):
             # the same size, but different box
-            with_box = im.resize(im.size, Image.BILINEAR, box)
+            with_box = im.resize(im.size, Image.Resampling.BILINEAR, box)
         with self.count(2):
-            cropped = im.crop(box).resize(im.size, Image.BILINEAR)
+            cropped = im.crop(box).resize(im.size, Image.Resampling.BILINEAR)
         assert_image_similar(with_box, cropped, 0.1)
 
     def test_box_vertical(self):
@@ -373,9 +384,9 @@ class TestCoreResamplePasses:
         box = (0, 20, im.size[0], im.size[1] - 20)
         with self.count(1):
             # the same size, but different box
-            with_box = im.resize(im.size, Image.BILINEAR, box)
+            with_box = im.resize(im.size, Image.Resampling.BILINEAR, box)
         with self.count(2):
-            cropped = im.crop(box).resize(im.size, Image.BILINEAR)
+            cropped = im.crop(box).resize(im.size, Image.Resampling.BILINEAR)
         assert_image_similar(with_box, cropped, 0.1)
 
 
@@ -388,7 +399,7 @@ class TestCoreResampleCoefficients:
             draw = ImageDraw.Draw(i)
             draw.rectangle((0, 0, i.size[0] // 2 - 1, 0), test_color)
 
-            px = i.resize((5, i.size[1]), Image.BICUBIC).load()
+            px = i.resize((5, i.size[1]), Image.Resampling.BICUBIC).load()
             if px[2, 0] != test_color // 2:
                 assert test_color // 2 == px[2, 0]
 
@@ -396,7 +407,7 @@ class TestCoreResampleCoefficients:
         # regression test for the wrong coefficients calculation
         # due to bug https://github.com/python-pillow/Pillow/issues/2161
         im = Image.new("RGBA", (1280, 1280), (0x20, 0x40, 0x60, 0xFF))
-        histogram = im.resize((256, 256), Image.BICUBIC).histogram()
+        histogram = im.resize((256, 256), Image.Resampling.BICUBIC).histogram()
 
         # first channel
         assert histogram[0x100 * 0 + 0x20] == 0x10000
@@ -409,46 +420,49 @@ class TestCoreResampleCoefficients:
 
 
 class TestCoreResampleBox:
-    def test_wrong_arguments(self):
+    @pytest.mark.parametrize(
+        "resample",
+        (
+            Image.Resampling.NEAREST,
+            Image.Resampling.BOX,
+            Image.Resampling.BILINEAR,
+            Image.Resampling.HAMMING,
+            Image.Resampling.BICUBIC,
+            Image.Resampling.LANCZOS,
+        ),
+    )
+    def test_wrong_arguments(self, resample):
         im = hopper()
-        for resample in (
-            Image.NEAREST,
-            Image.BOX,
-            Image.BILINEAR,
-            Image.HAMMING,
-            Image.BICUBIC,
-            Image.LANCZOS,
-        ):
-            im.resize((32, 32), resample, (0, 0, im.width, im.height))
-            im.resize((32, 32), resample, (20, 20, im.width, im.height))
-            im.resize((32, 32), resample, (20, 20, 20, 100))
-            im.resize((32, 32), resample, (20, 20, 100, 20))
+        im.resize((32, 32), resample, (0, 0, im.width, im.height))
+        im.resize((32, 32), resample, (20, 20, im.width, im.height))
+        im.resize((32, 32), resample, (20, 20, 20, 100))
+        im.resize((32, 32), resample, (20, 20, 100, 20))
 
-            with pytest.raises(TypeError, match="must be sequence of length 4"):
-                im.resize((32, 32), resample, (im.width, im.height))
+        with pytest.raises(TypeError, match="must be sequence of length 4"):
+            im.resize((32, 32), resample, (im.width, im.height))
 
-            with pytest.raises(ValueError, match="can't be negative"):
-                im.resize((32, 32), resample, (-20, 20, 100, 100))
-            with pytest.raises(ValueError, match="can't be negative"):
-                im.resize((32, 32), resample, (20, -20, 100, 100))
+        with pytest.raises(ValueError, match="can't be negative"):
+            im.resize((32, 32), resample, (-20, 20, 100, 100))
+        with pytest.raises(ValueError, match="can't be negative"):
+            im.resize((32, 32), resample, (20, -20, 100, 100))
 
-            with pytest.raises(ValueError, match="can't be empty"):
-                im.resize((32, 32), resample, (20.1, 20, 20, 100))
-            with pytest.raises(ValueError, match="can't be empty"):
-                im.resize((32, 32), resample, (20, 20.1, 100, 20))
-            with pytest.raises(ValueError, match="can't be empty"):
-                im.resize((32, 32), resample, (20.1, 20.1, 20, 20))
+        with pytest.raises(ValueError, match="can't be empty"):
+            im.resize((32, 32), resample, (20.1, 20, 20, 100))
+        with pytest.raises(ValueError, match="can't be empty"):
+            im.resize((32, 32), resample, (20, 20.1, 100, 20))
+        with pytest.raises(ValueError, match="can't be empty"):
+            im.resize((32, 32), resample, (20.1, 20.1, 20, 20))
 
-            with pytest.raises(ValueError, match="can't exceed"):
-                im.resize((32, 32), resample, (0, 0, im.width + 1, im.height))
-            with pytest.raises(ValueError, match="can't exceed"):
-                im.resize((32, 32), resample, (0, 0, im.width, im.height + 1))
+        with pytest.raises(ValueError, match="can't exceed"):
+            im.resize((32, 32), resample, (0, 0, im.width + 1, im.height))
+        with pytest.raises(ValueError, match="can't exceed"):
+            im.resize((32, 32), resample, (0, 0, im.width, im.height + 1))
 
     def resize_tiled(self, im, dst_size, xtiles, ytiles):
         def split_range(size, tiles):
             scale = size / tiles
             for i in range(tiles):
-                yield (int(round(scale * i)), int(round(scale * (i + 1))))
+                yield int(round(scale * i)), int(round(scale * (i + 1)))
 
         tiled = Image.new(im.mode, dst_size)
         scale = (im.size[0] / tiled.size[0], im.size[1] / tiled.size[1])
@@ -456,7 +470,7 @@ class TestCoreResampleBox:
         for y0, y1 in split_range(dst_size[1], ytiles):
             for x0, x1 in split_range(dst_size[0], xtiles):
                 box = (x0 * scale[0], y0 * scale[1], x1 * scale[0], y1 * scale[1])
-                tile = im.resize((x1 - x0, y1 - y0), Image.BICUBIC, box)
+                tile = im.resize((x1 - x0, y1 - y0), Image.Resampling.BICUBIC, box)
                 tiled.paste(tile, (x0, y0))
         return tiled
 
@@ -467,7 +481,7 @@ class TestCoreResampleBox:
         with Image.open("Tests/images/flower.jpg") as im:
             assert im.size == (480, 360)
             dst_size = (251, 188)
-            reference = im.resize(dst_size, Image.BICUBIC)
+            reference = im.resize(dst_size, Image.Resampling.BICUBIC)
 
             for tiles in [(1, 1), (3, 3), (9, 7), (100, 100)]:
                 tiled = self.resize_tiled(im, dst_size, *tiles)
@@ -483,26 +497,32 @@ class TestCoreResampleBox:
             assert im.size == (480, 360)
             dst_size = (48, 36)
             # Reference is cropped image resized to destination
-            reference = im.crop((0, 0, 473, 353)).resize(dst_size, Image.BICUBIC)
-            # Image.BOX emulates supersampling (480 / 8 = 60, 360 / 8 = 45)
-            supersampled = im.resize((60, 45), Image.BOX)
+            reference = im.crop((0, 0, 473, 353)).resize(
+                dst_size, Image.Resampling.BICUBIC
+            )
+            # Image.Resampling.BOX emulates supersampling (480 / 8 = 60, 360 / 8 = 45)
+            supersampled = im.resize((60, 45), Image.Resampling.BOX)
 
-        with_box = supersampled.resize(dst_size, Image.BICUBIC, (0, 0, 59.125, 44.125))
-        without_box = supersampled.resize(dst_size, Image.BICUBIC)
+        with_box = supersampled.resize(
+            dst_size, Image.Resampling.BICUBIC, (0, 0, 59.125, 44.125)
+        )
+        without_box = supersampled.resize(dst_size, Image.Resampling.BICUBIC)
 
         # error with box should be much smaller than without
         assert_image_similar(reference, with_box, 6)
         with pytest.raises(AssertionError, match=r"difference 29\."):
             assert_image_similar(reference, without_box, 5)
 
-    def test_formats(self):
-        for resample in [Image.NEAREST, Image.BILINEAR]:
-            for mode in ["RGB", "L", "RGBA", "LA", "I", ""]:
-                im = hopper(mode)
-                box = (20, 20, im.size[0] - 20, im.size[1] - 20)
-                with_box = im.resize((32, 32), resample, box)
-                cropped = im.crop(box).resize((32, 32), resample)
-                assert_image_similar(cropped, with_box, 0.4)
+    @pytest.mark.parametrize("mode", ("RGB", "L", "RGBA", "LA", "I", ""))
+    @pytest.mark.parametrize(
+        "resample", (Image.Resampling.NEAREST, Image.Resampling.BILINEAR)
+    )
+    def test_formats(self, mode, resample):
+        im = hopper(mode)
+        box = (20, 20, im.size[0] - 20, im.size[1] - 20)
+        with_box = im.resize((32, 32), resample, box)
+        cropped = im.crop(box).resize((32, 32), resample)
+        assert_image_similar(cropped, with_box, 0.4)
 
     def test_passthrough(self):
         # When no resize is required
@@ -514,7 +534,7 @@ class TestCoreResampleBox:
             ((40, 50), (10, 0, 50, 50)),
             ((40, 50), (10, 20, 50, 70)),
         ]:
-            res = im.resize(size, Image.LANCZOS, box)
+            res = im.resize(size, Image.Resampling.LANCZOS, box)
             assert res.size == size
             assert_image_equal(res, im.crop(box), f">>> {size} {box}")
 
@@ -528,50 +548,54 @@ class TestCoreResampleBox:
             ((40, 50), (10.4, 0.4, 50.4, 50.4)),
             ((40, 50), (10.4, 20.4, 50.4, 70.4)),
         ]:
-            res = im.resize(size, Image.LANCZOS, box)
+            res = im.resize(size, Image.Resampling.LANCZOS, box)
             assert res.size == size
             with pytest.raises(AssertionError, match=r"difference \d"):
                 # check that the difference at least that much
                 assert_image_similar(res, im.crop(box), 20, f">>> {size} {box}")
 
-    def test_skip_horizontal(self):
+    @pytest.mark.parametrize(
+        "flt", (Image.Resampling.NEAREST, Image.Resampling.BICUBIC)
+    )
+    def test_skip_horizontal(self, flt):
         # Can skip resize for one dimension
         im = hopper()
 
-        for flt in [Image.NEAREST, Image.BICUBIC]:
-            for size, box in [
-                ((40, 50), (0, 0, 40, 90)),
-                ((40, 50), (0, 20, 40, 90)),
-                ((40, 50), (10, 0, 50, 90)),
-                ((40, 50), (10, 20, 50, 90)),
-            ]:
-                res = im.resize(size, flt, box)
-                assert res.size == size
-                # Borders should be slightly different
-                assert_image_similar(
-                    res,
-                    im.crop(box).resize(size, flt),
-                    0.4,
-                    f">>> {size} {box} {flt}",
-                )
+        for size, box in [
+            ((40, 50), (0, 0, 40, 90)),
+            ((40, 50), (0, 20, 40, 90)),
+            ((40, 50), (10, 0, 50, 90)),
+            ((40, 50), (10, 20, 50, 90)),
+        ]:
+            res = im.resize(size, flt, box)
+            assert res.size == size
+            # Borders should be slightly different
+            assert_image_similar(
+                res,
+                im.crop(box).resize(size, flt),
+                0.4,
+                f">>> {size} {box} {flt}",
+            )
 
-    def test_skip_vertical(self):
+    @pytest.mark.parametrize(
+        "flt", (Image.Resampling.NEAREST, Image.Resampling.BICUBIC)
+    )
+    def test_skip_vertical(self, flt):
         # Can skip resize for one dimension
         im = hopper()
 
-        for flt in [Image.NEAREST, Image.BICUBIC]:
-            for size, box in [
-                ((40, 50), (0, 0, 90, 50)),
-                ((40, 50), (20, 0, 90, 50)),
-                ((40, 50), (0, 10, 90, 60)),
-                ((40, 50), (20, 10, 90, 60)),
-            ]:
-                res = im.resize(size, flt, box)
-                assert res.size == size
-                # Borders should be slightly different
-                assert_image_similar(
-                    res,
-                    im.crop(box).resize(size, flt),
-                    0.4,
-                    f">>> {size} {box} {flt}",
-                )
+        for size, box in [
+            ((40, 50), (0, 0, 90, 50)),
+            ((40, 50), (20, 0, 90, 50)),
+            ((40, 50), (0, 10, 90, 60)),
+            ((40, 50), (20, 10, 90, 60)),
+        ]:
+            res = im.resize(size, flt, box)
+            assert res.size == size
+            # Borders should be slightly different
+            assert_image_similar(
+                res,
+                im.crop(box).resize(size, flt),
+                0.4,
+                f">>> {size} {box} {flt}",
+            )
