@@ -1,3 +1,5 @@
+import pytest
+
 from PIL import Image
 
 from .helper import (
@@ -22,38 +24,38 @@ def rotate(im, mode, angle, center=None, translate=None):
         assert out.size != im.size
 
 
-def test_mode():
-    for mode in ("1", "P", "L", "RGB", "I", "F"):
-        im = hopper(mode)
-        rotate(im, mode, 45)
+@pytest.mark.parametrize("mode", ("1", "P", "L", "RGB", "I", "F"))
+def test_mode(mode):
+    im = hopper(mode)
+    rotate(im, mode, 45)
 
 
-def test_angle():
-    for angle in (0, 90, 180, 270):
-        with Image.open("Tests/images/test-card.png") as im:
-            rotate(im, im.mode, angle)
-
-        im = hopper()
-        assert_image_equal(im.rotate(angle), im.rotate(angle, expand=1))
-
-
-def test_zero():
-    for angle in (0, 45, 90, 180, 270):
-        im = Image.new("RGB", (0, 0))
+@pytest.mark.parametrize("angle", (0, 90, 180, 270))
+def test_angle(angle):
+    with Image.open("Tests/images/test-card.png") as im:
         rotate(im, im.mode, angle)
 
+    im = hopper()
+    assert_image_equal(im.rotate(angle), im.rotate(angle, expand=1))
+
+
+@pytest.mark.parametrize("angle", (0, 45, 90, 180, 270))
+def test_zero(angle):
+    im = Image.new("RGB", (0, 0))
+    rotate(im, im.mode, angle)
+
 
 def test_resample():
     # Target image creation, inspected by eye.
     # >>> im = Image.open('Tests/images/hopper.ppm')
-    # >>> im = im.rotate(45, resample=Image.BICUBIC, expand=True)
+    # >>> im = im.rotate(45, resample=Image.Resampling.BICUBIC, expand=True)
     # >>> im.save('Tests/images/hopper_45.png')
 
     with Image.open("Tests/images/hopper_45.png") as target:
         for (resample, epsilon) in (
-            (Image.NEAREST, 10),
-            (Image.BILINEAR, 5),
-            (Image.BICUBIC, 0),
+            (Image.Resampling.NEAREST, 10),
+            (Image.Resampling.BILINEAR, 5),
+            (Image.Resampling.BICUBIC, 0),
         ):
             im = hopper()
             im = im.rotate(45, resample=resample, expand=True)
@@ -62,7 +64,7 @@ def test_resample():
 
 def test_center_0():
     im = hopper()
-    im = im.rotate(45, center=(0, 0), resample=Image.BICUBIC)
+    im = im.rotate(45, center=(0, 0), resample=Image.Resampling.BICUBIC)
 
     with Image.open("Tests/images/hopper_45.png") as target:
         target_origin = target.size[1] / 2
@@ -73,7 +75,7 @@ def test_center_0():
 
 def test_center_14():
     im = hopper()
-    im = im.rotate(45, center=(14, 14), resample=Image.BICUBIC)
+    im = im.rotate(45, center=(14, 14), resample=Image.Resampling.BICUBIC)
 
     with Image.open("Tests/images/hopper_45.png") as target:
         target_origin = target.size[1] / 2 - 14
@@ -90,7 +92,7 @@ def test_translate():
             (target_origin, target_origin, target_origin + 128, target_origin + 128)
         )
 
-    im = im.rotate(45, translate=(5, 5), resample=Image.BICUBIC)
+    im = im.rotate(45, translate=(5, 5), resample=Image.Resampling.BICUBIC)
 
     assert_image_similar(im, target, 1)
 
