@@ -1,11 +1,13 @@
 import math
 
+import pytest
+
 from PIL import Image, ImageTransform
 
-from .helper import PillowTestCase, hopper
+from .helper import assert_image_equal, assert_image_similar, hopper
 
 
-class TestImageTransform(PillowTestCase):
+class TestImageTransform:
     def test_sanity(self):
         im = Image.new("L", (100, 100))
 
@@ -24,93 +26,108 @@ class TestImageTransform(PillowTestCase):
         comment = b"File written by Adobe Photoshop\xa8 4.0"
 
         with Image.open("Tests/images/hopper.gif") as im:
-            self.assertEqual(im.info["comment"], comment)
+            assert im.info["comment"] == comment
 
             transform = ImageTransform.ExtentTransform((0, 0, 0, 0))
             new_im = im.transform((100, 100), transform)
-        self.assertEqual(new_im.info["comment"], comment)
+        assert new_im.info["comment"] == comment
+
+    def test_palette(self):
+        with Image.open("Tests/images/hopper.gif") as im:
+            transformed = im.transform(
+                im.size, Image.Transform.AFFINE, [1, 0, 0, 0, 1, 0]
+            )
+            assert im.palette.palette == transformed.palette.palette
 
     def test_extent(self):
         im = hopper("RGB")
         (w, h) = im.size
         # fmt: off
-        transformed = im.transform(im.size, Image.EXTENT,
+        transformed = im.transform(im.size, Image.Transform.EXTENT,
                                    (0, 0,
                                     w//2, h//2),  # ul -> lr
-                                   Image.BILINEAR)
+                                   Image.Resampling.BILINEAR)
         # fmt: on
 
-        scaled = im.resize((w * 2, h * 2), Image.BILINEAR).crop((0, 0, w, h))
+        scaled = im.resize((w * 2, h * 2), Image.Resampling.BILINEAR).crop((0, 0, w, h))
 
         # undone -- precision?
-        self.assert_image_similar(transformed, scaled, 23)
+        assert_image_similar(transformed, scaled, 23)
 
     def test_quad(self):
         # one simple quad transform, equivalent to scale & crop upper left quad
         im = hopper("RGB")
         (w, h) = im.size
         # fmt: off
-        transformed = im.transform(im.size, Image.QUAD,
+        transformed = im.transform(im.size, Image.Transform.QUAD,
                                    (0, 0, 0, h//2,
                                     # ul -> ccw around quad:
                                     w//2, h//2, w//2, 0),
-                                   Image.BILINEAR)
+                                   Image.Resampling.BILINEAR)
         # fmt: on
 
         scaled = im.transform(
-            (w, h), Image.AFFINE, (0.5, 0, 0, 0, 0.5, 0), Image.BILINEAR
+            (w, h),
+            Image.Transform.AFFINE,
+            (0.5, 0, 0, 0, 0.5, 0),
+            Image.Resampling.BILINEAR,
         )
 
-        self.assert_image_equal(transformed, scaled)
+        assert_image_equal(transformed, scaled)
 
-    def test_fill(self):
-        for mode, pixel in [
-            ["RGB", (255, 0, 0)],
-            ["RGBA", (255, 0, 0, 255)],
-            ["LA", (76, 0)],
-        ]:
-            im = hopper(mode)
-            (w, h) = im.size
-            transformed = im.transform(
-                im.size,
-                Image.EXTENT,
-                (0, 0, w * 2, h * 2),
-                Image.BILINEAR,
-                fillcolor="red",
-            )
-
-            self.assertEqual(transformed.getpixel((w - 1, h - 1)), pixel)
+    @pytest.mark.parametrize(
+        "mode, expected_pixel",
+        (
+            ("RGB", (255, 0, 0)),
+            ("RGBA", (255, 0, 0, 255)),
+            ("LA", (76, 0)),
+        ),
+    )
+    def test_fill(self, mode, expected_pixel):
+        im = hopper(mode)
+        (w, h) = im.size
+        transformed = im.transform(
+            im.size,
+            Image.Transform.EXTENT,
+            (0, 0, w * 2, h * 2),
+            Image.Resampling.BILINEAR,
+            fillcolor="red",
+        )
+        assert transformed.getpixel((w - 1, h - 1)) == expected_pixel
 
     def test_mesh(self):
         # this should be a checkerboard of halfsized hoppers in ul, lr
         im = hopper("RGBA")
         (w, h) = im.size
         # fmt: off
-        transformed = im.transform(im.size, Image.MESH,
+        transformed = im.transform(im.size, Image.Transform.MESH,
                                    [((0, 0, w//2, h//2),  # box
                                     (0, 0, 0, h,
                                      w, h, w, 0)),  # ul -> ccw around quad
                                     ((w//2, h//2, w, h),  # box
                                     (0, 0, 0, h,
                                      w, h, w, 0))],  # ul -> ccw around quad
-                                   Image.BILINEAR)
+                                   Image.Resampling.BILINEAR)
         # fmt: on
 
         scaled = im.transform(
-            (w // 2, h // 2), Image.AFFINE, (2, 0, 0, 0, 2, 0), Image.BILINEAR
+            (w // 2, h // 2),
+            Image.Transform.AFFINE,
+            (2, 0, 0, 0, 2, 0),
+            Image.Resampling.BILINEAR,
         )
 
         checker = Image.new("RGBA", im.size)
         checker.paste(scaled, (0, 0))
         checker.paste(scaled, (w // 2, h // 2))
 
-        self.assert_image_equal(transformed, checker)
+        assert_image_equal(transformed, checker)
 
         # now, check to see that the extra area is (0, 0, 0, 0)
         blank = Image.new("RGBA", (w // 2, h // 2), (0, 0, 0, 0))
 
-        self.assert_image_equal(blank, transformed.crop((w // 2, 0, w, h // 2)))
-        self.assert_image_equal(blank, transformed.crop((0, h // 2, w // 2, h)))
+        assert_image_equal(blank, transformed.crop((w // 2, 0, w, h // 2)))
+        assert_image_equal(blank, transformed.crop((0, h // 2, w // 2, h)))
 
     def _test_alpha_premult(self, op):
         # create image with half white, half black,
@@ -126,21 +143,60 @@ class TestImageTransform(PillowTestCase):
         im_background.paste(im, (0, 0), im)
 
         hist = im_background.histogram()
-        self.assertEqual(40 * 10, hist[-1])
+        assert 40 * 10 == hist[-1]
 
     def test_alpha_premult_resize(self):
         def op(im, sz):
-            return im.resize(sz, Image.BILINEAR)
+            return im.resize(sz, Image.Resampling.BILINEAR)
 
         self._test_alpha_premult(op)
 
     def test_alpha_premult_transform(self):
         def op(im, sz):
             (w, h) = im.size
-            return im.transform(sz, Image.EXTENT, (0, 0, w, h), Image.BILINEAR)
+            return im.transform(
+                sz, Image.Transform.EXTENT, (0, 0, w, h), Image.Resampling.BILINEAR
+            )
 
         self._test_alpha_premult(op)
 
+    def _test_nearest(self, op, mode):
+        # create white image with half transparent,
+        # do op,
+        # the image should remain white with half transparent
+        transparent, opaque = {
+            "RGBA": ((255, 255, 255, 0), (255, 255, 255, 255)),
+            "LA": ((255, 0), (255, 255)),
+        }[mode]
+        im = Image.new(mode, (10, 10), transparent)
+        im2 = Image.new(mode, (5, 10), opaque)
+        im.paste(im2, (0, 0))
+
+        im = op(im, (40, 10))
+
+        colors = im.getcolors()
+        assert colors == [
+            (20 * 10, opaque),
+            (20 * 10, transparent),
+        ]
+
+    @pytest.mark.parametrize("mode", ("RGBA", "LA"))
+    def test_nearest_resize(self, mode):
+        def op(im, sz):
+            return im.resize(sz, Image.Resampling.NEAREST)
+
+        self._test_nearest(op, mode)
+
+    @pytest.mark.parametrize("mode", ("RGBA", "LA"))
+    def test_nearest_transform(self, mode):
+        def op(im, sz):
+            (w, h) = im.size
+            return im.transform(
+                sz, Image.Transform.EXTENT, (0, 0, w, h), Image.Resampling.NEAREST
+            )
+
+        self._test_nearest(op, mode)
+
     def test_blank_fill(self):
         # attempting to hit
         # https://github.com/python-pillow/Pillow/issues/254 reported
@@ -165,30 +221,34 @@ class TestImageTransform(PillowTestCase):
 
     def test_missing_method_data(self):
         with hopper() as im:
-            self.assertRaises(ValueError, im.transform, (100, 100), None)
+            with pytest.raises(ValueError):
+                im.transform((100, 100), None)
 
-    def test_unknown_resampling_filter(self):
+    @pytest.mark.parametrize("resample", (Image.Resampling.BOX, "unknown"))
+    def test_unknown_resampling_filter(self, resample):
         with hopper() as im:
             (w, h) = im.size
-            for resample in (Image.BOX, "unknown"):
-                self.assertRaises(
-                    ValueError,
-                    im.transform,
-                    (100, 100),
-                    Image.EXTENT,
-                    (0, 0, w, h),
-                    resample,
-                )
+            with pytest.raises(ValueError):
+                im.transform((100, 100), Image.Transform.EXTENT, (0, 0, w, h), resample)
 
 
-class TestImageTransformAffine(PillowTestCase):
-    transform = Image.AFFINE
+class TestImageTransformAffine:
+    transform = Image.Transform.AFFINE
 
     def _test_image(self):
         im = hopper("RGB")
         return im.crop((10, 20, im.width - 10, im.height - 20))
 
-    def _test_rotate(self, deg, transpose):
+    @pytest.mark.parametrize(
+        "deg, transpose",
+        (
+            (0, None),
+            (90, Image.Transpose.ROTATE_90),
+            (180, Image.Transpose.ROTATE_180),
+            (270, Image.Transpose.ROTATE_270),
+        ),
+    )
+    def test_rotate(self, deg, transpose):
         im = self._test_image()
 
         angle = -math.radians(deg)
@@ -210,85 +270,77 @@ class TestImageTransformAffine(PillowTestCase):
         else:
             transposed = im
 
-        for resample in [Image.NEAREST, Image.BILINEAR, Image.BICUBIC]:
+        for resample in [
+            Image.Resampling.NEAREST,
+            Image.Resampling.BILINEAR,
+            Image.Resampling.BICUBIC,
+        ]:
             transformed = im.transform(
                 transposed.size, self.transform, matrix, resample
             )
-            self.assert_image_equal(transposed, transformed)
+            assert_image_equal(transposed, transformed)
 
-    def test_rotate_0_deg(self):
-        self._test_rotate(0, None)
-
-    def test_rotate_90_deg(self):
-        self._test_rotate(90, Image.ROTATE_90)
-
-    def test_rotate_180_deg(self):
-        self._test_rotate(180, Image.ROTATE_180)
-
-    def test_rotate_270_deg(self):
-        self._test_rotate(270, Image.ROTATE_270)
-
-    def _test_resize(self, scale, epsilonscale):
+    @pytest.mark.parametrize(
+        "scale, epsilon_scale",
+        (
+            (1.1, 6.9),
+            (1.5, 5.5),
+            (2.0, 5.5),
+            (2.3, 3.7),
+            (2.5, 3.7),
+        ),
+    )
+    @pytest.mark.parametrize(
+        "resample,epsilon",
+        (
+            (Image.Resampling.NEAREST, 0),
+            (Image.Resampling.BILINEAR, 2),
+            (Image.Resampling.BICUBIC, 1),
+        ),
+    )
+    def test_resize(self, scale, epsilon_scale, resample, epsilon):
         im = self._test_image()
 
         size_up = int(round(im.width * scale)), int(round(im.height * scale))
         matrix_up = [1 / scale, 0, 0, 0, 1 / scale, 0, 0, 0]
         matrix_down = [scale, 0, 0, 0, scale, 0, 0, 0]
 
-        for resample, epsilon in [
-            (Image.NEAREST, 0),
-            (Image.BILINEAR, 2),
-            (Image.BICUBIC, 1),
-        ]:
-            transformed = im.transform(size_up, self.transform, matrix_up, resample)
-            transformed = transformed.transform(
-                im.size, self.transform, matrix_down, resample
-            )
-            self.assert_image_similar(transformed, im, epsilon * epsilonscale)
+        transformed = im.transform(size_up, self.transform, matrix_up, resample)
+        transformed = transformed.transform(
+            im.size, self.transform, matrix_down, resample
+        )
+        assert_image_similar(transformed, im, epsilon * epsilon_scale)
 
-    def test_resize_1_1x(self):
-        self._test_resize(1.1, 6.9)
-
-    def test_resize_1_5x(self):
-        self._test_resize(1.5, 5.5)
-
-    def test_resize_2_0x(self):
-        self._test_resize(2.0, 5.5)
-
-    def test_resize_2_3x(self):
-        self._test_resize(2.3, 3.7)
-
-    def test_resize_2_5x(self):
-        self._test_resize(2.5, 3.7)
-
-    def _test_translate(self, x, y, epsilonscale):
+    @pytest.mark.parametrize(
+        "x, y, epsilon_scale",
+        (
+            (0.1, 0, 3.7),
+            (0.6, 0, 9.1),
+            (50, 50, 0),
+        ),
+    )
+    @pytest.mark.parametrize(
+        "resample, epsilon",
+        (
+            (Image.Resampling.NEAREST, 0),
+            (Image.Resampling.BILINEAR, 1.5),
+            (Image.Resampling.BICUBIC, 1),
+        ),
+    )
+    def test_translate(self, x, y, epsilon_scale, resample, epsilon):
         im = self._test_image()
 
         size_up = int(round(im.width + x)), int(round(im.height + y))
         matrix_up = [1, 0, -x, 0, 1, -y, 0, 0]
         matrix_down = [1, 0, x, 0, 1, y, 0, 0]
 
-        for resample, epsilon in [
-            (Image.NEAREST, 0),
-            (Image.BILINEAR, 1.5),
-            (Image.BICUBIC, 1),
-        ]:
-            transformed = im.transform(size_up, self.transform, matrix_up, resample)
-            transformed = transformed.transform(
-                im.size, self.transform, matrix_down, resample
-            )
-            self.assert_image_similar(transformed, im, epsilon * epsilonscale)
-
-    def test_translate_0_1(self):
-        self._test_translate(0.1, 0, 3.7)
-
-    def test_translate_0_6(self):
-        self._test_translate(0.6, 0, 9.1)
-
-    def test_translate_50(self):
-        self._test_translate(50, 50, 0)
+        transformed = im.transform(size_up, self.transform, matrix_up, resample)
+        transformed = transformed.transform(
+            im.size, self.transform, matrix_down, resample
+        )
+        assert_image_similar(transformed, im, epsilon * epsilon_scale)
 
 
 class TestImageTransformPerspective(TestImageTransformAffine):
     # Repeat all tests for AFFINE transformations with PERSPECTIVE
-    transform = Image.PERSPECTIVE
+    transform = Image.Transform.PERSPECTIVE
