@@ -6,10 +6,8 @@ from PIL import Image, ImageMath
 def pixel(im):
     if hasattr(im, "im"):
         return f"{im.mode} {repr(im.getpixel((0, 0)))}"
-    else:
-        if isinstance(im, int):
-            return int(im)  # hack to deal with booleans
-        print(im)
+    if isinstance(im, int):
+        return int(im)  # hack to deal with booleans
 
 
 A = Image.new("L", (1, 1), 1)
