@@ -1,33 +1,88 @@
 import pytest
+
 from PIL import Image, ImagePalette
 
-from .helper import assert_image_equal
+from .helper import assert_image_equal, assert_image_equal_tofile
 
 
 def test_sanity():
 
-    ImagePalette.ImagePalette("RGB", list(range(256)) * 3)
-    with pytest.raises(ValueError):
-        ImagePalette.ImagePalette("RGB", list(range(256)) * 2)
+    palette = ImagePalette.ImagePalette("RGB", list(range(256)) * 3)
+    assert len(palette.colors) == 256
+
+    with pytest.warns(DeprecationWarning):
+        with pytest.raises(ValueError):
+            ImagePalette.ImagePalette("RGB", list(range(256)) * 3, 10)
+
+
+def test_reload():
+    with Image.open("Tests/images/hopper.gif") as im:
+        original = im.copy()
+        im.palette.dirty = 1
+        assert_image_equal(im.convert("RGB"), original.convert("RGB"))
 
 
 def test_getcolor():
 
     palette = ImagePalette.ImagePalette()
+    assert len(palette.palette) == 0
+    assert len(palette.colors) == 0
 
     test_map = {}
     for i in range(256):
         test_map[palette.getcolor((i, i, i))] = i
-
     assert len(test_map) == 256
+
+    # Colors can be converted between RGB and RGBA
+    rgba_palette = ImagePalette.ImagePalette("RGBA")
+    assert rgba_palette.getcolor((0, 0, 0)) == rgba_palette.getcolor((0, 0, 0, 255))
+
+    assert palette.getcolor((0, 0, 0)) == palette.getcolor((0, 0, 0, 255))
+
+    # An error is raised when the palette is full
     with pytest.raises(ValueError):
         palette.getcolor((1, 2, 3))
+    # But not if the image is not using one of the palette entries
+    palette.getcolor((1, 2, 3), image=Image.new("P", (1, 1)))
 
     # Test unknown color specifier
     with pytest.raises(ValueError):
         palette.getcolor("unknown")
 
 
+def test_getcolor_rgba_color_rgb_palette():
+    palette = ImagePalette.ImagePalette("RGB")
+
+    # Opaque RGBA colors are converted
+    assert palette.getcolor((0, 0, 0, 255)) == palette.getcolor((0, 0, 0))
+
+    with pytest.raises(ValueError):
+        palette.getcolor((0, 0, 0, 128))
+
+
+@pytest.mark.parametrize(
+    "index, palette",
+    [
+        # Test when the palette is not full
+        (0, ImagePalette.ImagePalette()),
+        # Test when the palette is full
+        (255, ImagePalette.ImagePalette("RGB", list(range(256)) * 3)),
+    ],
+)
+def test_getcolor_not_special(index, palette):
+    im = Image.new("P", (1, 1))
+
+    # Do not use transparency index as a new color
+    im.info["transparency"] = index
+    index1 = palette.getcolor((0, 0, 0), im)
+    assert index1 != index
+
+    # Do not use background index as a new color
+    im.info["background"] = index1
+    index2 = palette.getcolor((0, 0, 1), im)
+    assert index2 not in (index, index1)
+
+
 def test_file(tmp_path):
 
     palette = ImagePalette.ImagePalette("RGB", list(range(256)) * 3)
@@ -115,7 +170,7 @@ def test_getdata():
     mode, data_out = palette.getdata()
 
     # Assert
-    assert mode == "RGB;L"
+    assert mode == "RGB"
 
 
 def test_rawmode_getdata():
@@ -140,8 +195,7 @@ def test_2bit_palette(tmp_path):
     img.putpalette(b"\xFF\x00\x00\x00\xFF\x00\x00\x00\xFF")  # RGB
     img.save(outfile, format="PNG")
 
-    with Image.open(outfile) as reloaded:
-        assert_image_equal(img, reloaded)
+    assert_image_equal_tofile(img, outfile)
 
 
 def test_invalid_palette():
