@@ -1,9 +1,14 @@
+import warnings
+
 import pytest
 
-from PIL import ImageQt
-
 from .helper import assert_image_similar, hopper
 
+with warnings.catch_warnings() as w:
+    warnings.simplefilter("ignore", category=DeprecationWarning)
+    from PIL import ImageQt
+
+
 pytestmark = pytest.mark.skipif(
     not ImageQt.qt_is_installed, reason="Qt bindings are not installed"
 )
@@ -30,10 +35,10 @@ def test_rgb():
 
     def checkrgb(r, g, b):
         val = ImageQt.rgb(r, g, b)
-        val = val % 2 ** 24  # drop the alpha
+        val = val % 2**24  # drop the alpha
         assert val >> 16 == r
-        assert ((val >> 8) % 2 ** 8) == g
-        assert val % 2 ** 8 == b
+        assert ((val >> 8) % 2**8) == g
+        assert val % 2**8 == b
 
     checkrgb(0, 0, 0)
     checkrgb(255, 0, 0)
@@ -56,7 +61,5 @@ def test_image():
 
 
 def test_closed_file():
-    with pytest.warns(None) as record:
+    with warnings.catch_warnings():
         ImageQt.ImageQt("Tests/images/hopper.gif")
-
-    assert not record
