@@ -1,10 +1,9 @@
 import locale
-import unittest
+
+import pytest
 
 from PIL import Image
 
-from .helper import PillowTestCase
-
 # ref https://github.com/python-pillow/Pillow/issues/272
 # on windows, in polish locale:
 
@@ -23,17 +22,16 @@ from .helper import PillowTestCase
 path = "Tests/images/hopper.jpg"
 
 
-class TestLocale(PillowTestCase):
-    def test_sanity(self):
+def test_sanity():
+    with Image.open(path):
+        pass
+    try:
+        locale.setlocale(locale.LC_ALL, "polish")
+    except locale.Error:
+        pytest.skip("Polish locale not available")
+
+    try:
         with Image.open(path):
             pass
-        try:
-            locale.setlocale(locale.LC_ALL, "polish")
-        except locale.Error:
-            unittest.skip("Polish locale not available")
-
-        try:
-            with Image.open(path):
-                pass
-        finally:
-            locale.setlocale(locale.LC_ALL, (None, None))
+    finally:
+        locale.setlocale(locale.LC_ALL, (None, None))
