@@ -1,3 +1,5 @@
+import pytest
+
 from PIL import Image
 
 from .helper import hopper
@@ -20,65 +22,56 @@ def verify(im1):
             ), f"got {repr(p1)} from mode {im1.mode} at {xy}, expected {repr(p2)}"
 
 
-def test_basic(tmp_path):
+@pytest.mark.parametrize("mode", ("L", "I;16", "I;16B", "I;16L", "I"))
+def test_basic(tmp_path, mode):
     # PIL 1.1 has limited support for 16-bit image data.  Check that
     # create/copy/transform and save works as expected.
 
-    def basic(mode):
+    im_in = original.convert(mode)
+    verify(im_in)
 
-        imIn = original.convert(mode)
-        verify(imIn)
+    w, h = im_in.size
 
-        w, h = imIn.size
+    im_out = im_in.copy()
+    verify(im_out)  # copy
 
-        imOut = imIn.copy()
-        verify(imOut)  # copy
+    im_out = im_in.transform((w, h), Image.Transform.EXTENT, (0, 0, w, h))
+    verify(im_out)  # transform
 
-        imOut = imIn.transform((w, h), Image.EXTENT, (0, 0, w, h))
-        verify(imOut)  # transform
+    filename = str(tmp_path / "temp.im")
+    im_in.save(filename)
 
-        filename = str(tmp_path / "temp.im")
-        imIn.save(filename)
+    with Image.open(filename) as im_out:
 
-        with Image.open(filename) as imOut:
+        verify(im_in)
+        verify(im_out)
 
-            verify(imIn)
-            verify(imOut)
+    im_out = im_in.crop((0, 0, w, h))
+    verify(im_out)
 
-        imOut = imIn.crop((0, 0, w, h))
-        verify(imOut)
+    im_out = Image.new(mode, (w, h), None)
+    im_out.paste(im_in.crop((0, 0, w // 2, h)), (0, 0))
+    im_out.paste(im_in.crop((w // 2, 0, w, h)), (w // 2, 0))
 
-        imOut = Image.new(mode, (w, h), None)
-        imOut.paste(imIn.crop((0, 0, w // 2, h)), (0, 0))
-        imOut.paste(imIn.crop((w // 2, 0, w, h)), (w // 2, 0))
+    verify(im_in)
+    verify(im_out)
 
-        verify(imIn)
-        verify(imOut)
+    im_in = Image.new(mode, (1, 1), 1)
+    assert im_in.getpixel((0, 0)) == 1
 
-        imIn = Image.new(mode, (1, 1), 1)
-        assert imIn.getpixel((0, 0)) == 1
+    im_in.putpixel((0, 0), 2)
+    assert im_in.getpixel((0, 0)) == 2
 
-        imIn.putpixel((0, 0), 2)
-        assert imIn.getpixel((0, 0)) == 2
+    if mode == "L":
+        maximum = 255
+    else:
+        maximum = 32767
 
-        if mode == "L":
-            maximum = 255
-        else:
-            maximum = 32767
+    im_in = Image.new(mode, (1, 1), 256)
+    assert im_in.getpixel((0, 0)) == min(256, maximum)
 
-        imIn = Image.new(mode, (1, 1), 256)
-        assert imIn.getpixel((0, 0)) == min(256, maximum)
-
-        imIn.putpixel((0, 0), 512)
-        assert imIn.getpixel((0, 0)) == min(512, maximum)
-
-    basic("L")
-
-    basic("I;16")
-    basic("I;16B")
-    basic("I;16L")
-
-    basic("I")
+    im_in.putpixel((0, 0), 512)
+    assert im_in.getpixel((0, 0)) == min(512, maximum)
 
 
 def test_tobytes():
