@@ -18,7 +18,10 @@ import os
 import tempfile
 
 from . import Image, ImageFile
-from ._binary import i8, i16be as i16, i32be as i32, o8
+from ._binary import i8
+from ._binary import i16be as i16
+from ._binary import i32be as i32
+from ._binary import o8
 
 COMPRESSION = {1: "raw", 5: "jpeg"}
 
@@ -59,22 +62,24 @@ class IptcImageFile(ImageFile.ImageFile):
         if not len(s):
             return None, 0
 
-        tag = i8(s[1]), i8(s[2])
+        tag = s[1], s[2]
 
         # syntax
-        if i8(s[0]) != 0x1C or tag[0] < 1 or tag[0] > 9:
-            raise SyntaxError("invalid IPTC/NAA file")
+        if s[0] != 0x1C or tag[0] < 1 or tag[0] > 9:
+            msg = "invalid IPTC/NAA file"
+            raise SyntaxError(msg)
 
         # field size
-        size = i8(s[3])
+        size = s[3]
         if size > 132:
-            raise OSError("illegal field length in IPTC/NAA file")
+            msg = "illegal field length in IPTC/NAA file"
+            raise OSError(msg)
         elif size == 128:
             size = 0
         elif size > 128:
             size = i(self.fp.read(size - 128))
         else:
-            size = i16(s[3:])
+            size = i16(s, 3)
 
         return tag, size
 
@@ -119,7 +124,8 @@ class IptcImageFile(ImageFile.ImageFile):
         try:
             compression = COMPRESSION[self.getint((3, 120))]
         except KeyError as e:
-            raise OSError("Unknown IPTC image compression") from e
+            msg = "Unknown IPTC image compression"
+            raise OSError(msg) from e
 
         # tile
         if tag == (8, 10):
@@ -181,9 +187,10 @@ def getiptcinfo(im):
     :returns: A dictionary containing IPTC information, or None if
         no IPTC information block was found.
     """
-    from . import TiffImagePlugin, JpegImagePlugin
     import io
 
+    from . import JpegImagePlugin, TiffImagePlugin
+
     data = None
 
     if isinstance(im, IptcImageFile):
