@@ -25,22 +25,25 @@
 # See the README file for information on usage and redistribution.
 #
 
+import io
 import logging
+
 from . import Image, ImageFile, ImagePalette
-from ._binary import i8, i16le as i16, o8, o16le as o16
+from ._binary import i16le as i16
+from ._binary import o8
+from ._binary import o16le as o16
 
 logger = logging.getLogger(__name__)
 
-__version__ = "0.6"
-
 
 def _accept(prefix):
-    return i8(prefix[0]) == 10 and i8(prefix[1]) in [0, 2, 3, 5]
+    return prefix[0] == 10 and prefix[1] in [0, 2, 3, 5]
 
 
 ##
 # Image plugin for Paintbrush images.
 
+
 class PcxImageFile(ImageFile.ImageFile):
 
     format = "PCX"
@@ -51,21 +54,28 @@ class PcxImageFile(ImageFile.ImageFile):
         # header
         s = self.fp.read(128)
         if not _accept(s):
-            raise SyntaxError("not a PCX file")
+            msg = "not a PCX file"
+            raise SyntaxError(msg)
 
         # image
-        bbox = i16(s, 4), i16(s, 6), i16(s, 8)+1, i16(s, 10)+1
+        bbox = i16(s, 4), i16(s, 6), i16(s, 8) + 1, i16(s, 10) + 1
         if bbox[2] <= bbox[0] or bbox[3] <= bbox[1]:
-            raise SyntaxError("bad PCX image size")
+            msg = "bad PCX image size"
+            raise SyntaxError(msg)
         logger.debug("BBox: %s %s %s %s", *bbox)
 
         # format
-        version = i8(s[1])
-        bits = i8(s[3])
-        planes = i8(s[65])
-        stride = i16(s, 66)
-        logger.debug("PCX version %s, bits %s, planes %s, stride %s",
-                     version, bits, planes, stride)
+        version = s[1]
+        bits = s[3]
+        planes = s[65]
+        provided_stride = i16(s, 66)
+        logger.debug(
+            "PCX version %s, bits %s, planes %s, stride %s",
+            version,
+            bits,
+            planes,
+            provided_stride,
+        )
 
         self.info["dpi"] = i16(s, 12), i16(s, 14)
 
@@ -80,12 +90,12 @@ class PcxImageFile(ImageFile.ImageFile):
         elif version == 5 and bits == 8 and planes == 1:
             mode = rawmode = "L"
             # FIXME: hey, this doesn't work with the incremental loader !!!
-            self.fp.seek(-769, 2)
+            self.fp.seek(-769, io.SEEK_END)
             s = self.fp.read(769)
-            if len(s) == 769 and i8(s[0]) == 12:
+            if len(s) == 769 and s[0] == 12:
                 # check if the palette is linear greyscale
                 for i in range(256):
-                    if s[i*3+1:i*3+4] != o8(i)*3:
+                    if s[i * 3 + 1 : i * 3 + 4] != o8(i) * 3:
                         mode = rawmode = "P"
                         break
                 if mode == "P":
@@ -97,16 +107,28 @@ class PcxImageFile(ImageFile.ImageFile):
             rawmode = "RGB;L"
 
         else:
-            raise IOError("unknown PCX mode")
+            msg = "unknown PCX mode"
+            raise OSError(msg)
 
         self.mode = mode
-        self._size = bbox[2]-bbox[0], bbox[3]-bbox[1]
+        self._size = bbox[2] - bbox[0], bbox[3] - bbox[1]
+
+        # Don't trust the passed in stride.
+        # Calculate the approximate position for ourselves.
+        # CVE-2020-35653
+        stride = (self._size[0] * bits + 7) // 8
+
+        # While the specification states that this must be even,
+        # not all images follow this
+        if provided_stride != stride:
+            stride += stride % 2
 
         bbox = (0, 0) + self.size
         logger.debug("size: %sx%s", *self.size)
 
         self.tile = [("pcx", bbox, self.fp.tell(), (rawmode, planes * stride))]
 
+
 # --------------------------------------------------------------------
 # save PCX files
 
@@ -124,8 +146,9 @@ def _save(im, fp, filename):
 
     try:
         version, bits, planes, rawmode = SAVE[im.mode]
-    except KeyError:
-        raise ValueError("Cannot save %s images as PCX" % im.mode)
+    except KeyError as e:
+        msg = f"Cannot save {im.mode} images as PCX"
+        raise ValueError(msg) from e
 
     # bytes per plane
     stride = (im.size[0] * bits + 7) // 8
@@ -135,8 +158,12 @@ def _save(im, fp, filename):
     # Ideally it should be passed in in the state, but the bytes value
     # gets overwritten.
 
-    logger.debug("PcxImagePlugin._save: xwidth: %d, bits: %d, stride: %d",
-                 im.size[0], bits, stride)
+    logger.debug(
+        "PcxImagePlugin._save: xwidth: %d, bits: %d, stride: %d",
+        im.size[0],
+        bits,
+        stride,
+    )
 
     # under windows, we could determine the current screen size with
     # "Image.core.display_mode()[1]", but I think that's overkill...
@@ -147,27 +174,43 @@ def _save(im, fp, filename):
 
     # PCX header
     fp.write(
-        o8(10) + o8(version) + o8(1) + o8(bits) + o16(0) +
-        o16(0) + o16(im.size[0]-1) + o16(im.size[1]-1) + o16(dpi[0]) +
-        o16(dpi[1]) + b"\0"*24 + b"\xFF"*24 + b"\0" + o8(planes) +
-        o16(stride) + o16(1) + o16(screen[0]) + o16(screen[1]) +
-        b"\0"*54
-        )
+        o8(10)
+        + o8(version)
+        + o8(1)
+        + o8(bits)
+        + o16(0)
+        + o16(0)
+        + o16(im.size[0] - 1)
+        + o16(im.size[1] - 1)
+        + o16(dpi[0])
+        + o16(dpi[1])
+        + b"\0" * 24
+        + b"\xFF" * 24
+        + b"\0"
+        + o8(planes)
+        + o16(stride)
+        + o16(1)
+        + o16(screen[0])
+        + o16(screen[1])
+        + b"\0" * 54
+    )
 
     assert fp.tell() == 128
 
-    ImageFile._save(im, fp, [("pcx", (0, 0)+im.size, 0,
-                              (rawmode, bits*planes))])
+    ImageFile._save(im, fp, [("pcx", (0, 0) + im.size, 0, (rawmode, bits * planes))])
 
     if im.mode == "P":
         # colour palette
         fp.write(o8(12))
-        fp.write(im.im.getpalette("RGB", "RGB"))  # 768 bytes
+        palette = im.im.getpalette("RGB", "RGB")
+        palette += b"\x00" * (768 - len(palette))
+        fp.write(palette)  # 768 bytes
     elif im.mode == "L":
         # greyscale palette
         fp.write(o8(12))
         for i in range(256):
-            fp.write(o8(i)*3)
+            fp.write(o8(i) * 3)
+
 
 # --------------------------------------------------------------------
 # registry
@@ -177,3 +220,5 @@ Image.register_open(PcxImageFile.format, PcxImageFile, _accept)
 Image.register_save(PcxImageFile.format, _save)
 
 Image.register_extension(PcxImageFile.format, ".pcx")
+
+Image.register_mime(PcxImageFile.format, "image/x-pcx")
