@@ -138,9 +138,10 @@ class XrefTable:
         elif key in self.deleted_entries:
             generation = self.deleted_entries[key]
         else:
-            raise IndexError(
+            msg = (
                 "object ID " + str(key) + " cannot be deleted because it doesn't exist"
             )
+            raise IndexError(msg)
 
     def __contains__(self, key):
         return key in self.existing_entries or key in self.new_entries
@@ -314,9 +315,8 @@ class PdfStream:
                 expected_length = self.dictionary.Length
             return zlib.decompress(self.buf, bufsize=int(expected_length))
         else:
-            raise NotImplementedError(
-                f"stream filter {repr(self.dictionary.Filter)} unknown/unsupported"
-            )
+            msg = f"stream filter {repr(self.dictionary.Filter)} unknown/unsupported"
+            raise NotImplementedError(msg)
 
 
 def pdf_repr(x):
@@ -358,7 +358,8 @@ class PdfParser:
 
     def __init__(self, filename=None, f=None, buf=None, start_offset=0, mode="rb"):
         if buf and f:
-            raise RuntimeError("specify buf or f or filename, but not both buf and f")
+            msg = "specify buf or f or filename, but not both buf and f"
+            raise RuntimeError(msg)
         self.filename = filename
         self.buf = buf
         self.f = f
@@ -576,42 +577,42 @@ class PdfParser:
             self.xref_table[reference.object_id] = (offset, 0)
         return reference
 
-    delimiter = br"[][()<>{}/%]"
-    delimiter_or_ws = br"[][()<>{}/%\000\011\012\014\015\040]"
-    whitespace = br"[\000\011\012\014\015\040]"
-    whitespace_or_hex = br"[\000\011\012\014\015\0400-9a-fA-F]"
+    delimiter = rb"[][()<>{}/%]"
+    delimiter_or_ws = rb"[][()<>{}/%\000\011\012\014\015\040]"
+    whitespace = rb"[\000\011\012\014\015\040]"
+    whitespace_or_hex = rb"[\000\011\012\014\015\0400-9a-fA-F]"
     whitespace_optional = whitespace + b"*"
     whitespace_mandatory = whitespace + b"+"
     # No "\012" aka "\n" or "\015" aka "\r":
-    whitespace_optional_no_nl = br"[\000\011\014\040]*"
-    newline_only = br"[\r\n]+"
+    whitespace_optional_no_nl = rb"[\000\011\014\040]*"
+    newline_only = rb"[\r\n]+"
     newline = whitespace_optional_no_nl + newline_only + whitespace_optional_no_nl
     re_trailer_end = re.compile(
         whitespace_mandatory
-        + br"trailer"
+        + rb"trailer"
         + whitespace_optional
-        + br"\<\<(.*\>\>)"
+        + rb"<<(.*>>)"
         + newline
-        + br"startxref"
+        + rb"startxref"
         + newline
-        + br"([0-9]+)"
+        + rb"([0-9]+)"
         + newline
-        + br"%%EOF"
+        + rb"%%EOF"
         + whitespace_optional
-        + br"$",
+        + rb"$",
         re.DOTALL,
     )
     re_trailer_prev = re.compile(
         whitespace_optional
-        + br"trailer"
+        + rb"trailer"
         + whitespace_optional
-        + br"\<\<(.*?\>\>)"
+        + rb"<<(.*?>>)"
         + newline
-        + br"startxref"
+        + rb"startxref"
         + newline
-        + br"([0-9]+)"
+        + rb"([0-9]+)"
         + newline
-        + br"%%EOF"
+        + rb"%%EOF"
         + whitespace_optional,
         re.DOTALL,
     )
@@ -655,12 +656,12 @@ class PdfParser:
     re_whitespace_optional = re.compile(whitespace_optional)
     re_name = re.compile(
         whitespace_optional
-        + br"/([!-$&'*-.0-;=?-Z\\^-z|~]+)(?="
+        + rb"/([!-$&'*-.0-;=?-Z\\^-z|~]+)(?="
         + delimiter_or_ws
-        + br")"
+        + rb")"
     )
-    re_dict_start = re.compile(whitespace_optional + br"\<\<")
-    re_dict_end = re.compile(whitespace_optional + br"\>\>" + whitespace_optional)
+    re_dict_start = re.compile(whitespace_optional + rb"<<")
+    re_dict_end = re.compile(whitespace_optional + rb">>" + whitespace_optional)
 
     @classmethod
     def interpret_trailer(cls, trailer_data):
@@ -689,7 +690,7 @@ class PdfParser:
         )
         return trailer
 
-    re_hashes_in_name = re.compile(br"([^#]*)(#([0-9a-fA-F]{2}))?")
+    re_hashes_in_name = re.compile(rb"([^#]*)(#([0-9a-fA-F]{2}))?")
 
     @classmethod
     def interpret_name(cls, raw, as_text=False):
@@ -704,53 +705,53 @@ class PdfParser:
         else:
             return bytes(name)
 
-    re_null = re.compile(whitespace_optional + br"null(?=" + delimiter_or_ws + br")")
-    re_true = re.compile(whitespace_optional + br"true(?=" + delimiter_or_ws + br")")
-    re_false = re.compile(whitespace_optional + br"false(?=" + delimiter_or_ws + br")")
+    re_null = re.compile(whitespace_optional + rb"null(?=" + delimiter_or_ws + rb")")
+    re_true = re.compile(whitespace_optional + rb"true(?=" + delimiter_or_ws + rb")")
+    re_false = re.compile(whitespace_optional + rb"false(?=" + delimiter_or_ws + rb")")
     re_int = re.compile(
-        whitespace_optional + br"([-+]?[0-9]+)(?=" + delimiter_or_ws + br")"
+        whitespace_optional + rb"([-+]?[0-9]+)(?=" + delimiter_or_ws + rb")"
     )
     re_real = re.compile(
         whitespace_optional
-        + br"([-+]?([0-9]+\.[0-9]*|[0-9]*\.[0-9]+))(?="
+        + rb"([-+]?([0-9]+\.[0-9]*|[0-9]*\.[0-9]+))(?="
         + delimiter_or_ws
-        + br")"
+        + rb")"
     )
-    re_array_start = re.compile(whitespace_optional + br"\[")
-    re_array_end = re.compile(whitespace_optional + br"]")
+    re_array_start = re.compile(whitespace_optional + rb"\[")
+    re_array_end = re.compile(whitespace_optional + rb"]")
     re_string_hex = re.compile(
-        whitespace_optional + br"\<(" + whitespace_or_hex + br"*)\>"
+        whitespace_optional + rb"<(" + whitespace_or_hex + rb"*)>"
     )
-    re_string_lit = re.compile(whitespace_optional + br"\(")
+    re_string_lit = re.compile(whitespace_optional + rb"\(")
     re_indirect_reference = re.compile(
         whitespace_optional
-        + br"([-+]?[0-9]+)"
+        + rb"([-+]?[0-9]+)"
         + whitespace_mandatory
-        + br"([-+]?[0-9]+)"
+        + rb"([-+]?[0-9]+)"
         + whitespace_mandatory
-        + br"R(?="
+        + rb"R(?="
         + delimiter_or_ws
-        + br")"
+        + rb")"
     )
     re_indirect_def_start = re.compile(
         whitespace_optional
-        + br"([-+]?[0-9]+)"
+        + rb"([-+]?[0-9]+)"
         + whitespace_mandatory
-        + br"([-+]?[0-9]+)"
+        + rb"([-+]?[0-9]+)"
         + whitespace_mandatory
-        + br"obj(?="
+        + rb"obj(?="
         + delimiter_or_ws
-        + br")"
+        + rb")"
     )
     re_indirect_def_end = re.compile(
-        whitespace_optional + br"endobj(?=" + delimiter_or_ws + br")"
+        whitespace_optional + rb"endobj(?=" + delimiter_or_ws + rb")"
     )
     re_comment = re.compile(
-        br"(" + whitespace_optional + br"%[^\r\n]*" + newline + br")*"
+        rb"(" + whitespace_optional + rb"%[^\r\n]*" + newline + rb")*"
     )
-    re_stream_start = re.compile(whitespace_optional + br"stream\r?\n")
+    re_stream_start = re.compile(whitespace_optional + rb"stream\r?\n")
     re_stream_end = re.compile(
-        whitespace_optional + br"endstream(?=" + delimiter_or_ws + br")"
+        whitespace_optional + rb"endstream(?=" + delimiter_or_ws + rb")"
     )
 
     @classmethod
@@ -816,10 +817,10 @@ class PdfParser:
                 try:
                     stream_len = int(result[b"Length"])
                 except (TypeError, KeyError, ValueError) as e:
-                    raise PdfFormatError(
-                        "bad or missing Length in stream dict (%r)"
-                        % result.get(b"Length", None)
-                    ) from e
+                    msg = "bad or missing Length in stream dict (%r)" % result.get(
+                        b"Length", None
+                    )
+                    raise PdfFormatError(msg) from e
                 stream_data = data[m.end() : m.end() + stream_len]
                 m = cls.re_stream_end.match(data, m.end() + stream_len)
                 check_format_condition(m, "stream end not found")
@@ -873,10 +874,11 @@ class PdfParser:
         if m:
             return cls.get_literal_string(data, m.end())
         # return None, offset  # fallback (only for debugging)
-        raise PdfFormatError("unrecognized object: " + repr(data[offset : offset + 32]))
+        msg = "unrecognized object: " + repr(data[offset : offset + 32])
+        raise PdfFormatError(msg)
 
     re_lit_str_token = re.compile(
-        br"(\\[nrtbf()\\])|(\\[0-9]{1,3})|(\\(\r\n|\r|\n))|(\r\n|\r|\n)|(\()|(\))"
+        rb"(\\[nrtbf()\\])|(\\[0-9]{1,3})|(\\(\r\n|\r|\n))|(\r\n|\r|\n)|(\()|(\))"
     )
     escaped_chars = {
         b"n": b"\n",
@@ -920,18 +922,19 @@ class PdfParser:
                 result.extend(b")")
                 nesting_depth -= 1
             offset = m.end()
-        raise PdfFormatError("unfinished literal string")
+        msg = "unfinished literal string"
+        raise PdfFormatError(msg)
 
-    re_xref_section_start = re.compile(whitespace_optional + br"xref" + newline)
+    re_xref_section_start = re.compile(whitespace_optional + rb"xref" + newline)
     re_xref_subsection_start = re.compile(
         whitespace_optional
-        + br"([0-9]+)"
+        + rb"([0-9]+)"
         + whitespace_mandatory
-        + br"([0-9]+)"
+        + rb"([0-9]+)"
         + whitespace_optional
         + newline_only
     )
-    re_xref_entry = re.compile(br"([0-9]{10}) ([0-9]{5}) ([fn])( \r| \n|\r\n)")
+    re_xref_entry = re.compile(rb"([0-9]{10}) ([0-9]{5}) ([fn])( \r| \n|\r\n)")
 
     def read_xref_table(self, xref_section_offset):
         subsection_found = False
