@@ -73,7 +73,8 @@ class TgaImageFile(ImageFile.ImageFile):
             or self.size[1] <= 0
             or depth not in (1, 8, 16, 24, 32)
         ):
-            raise SyntaxError("not a TGA file")
+            msg = "not a TGA file"
+            raise SyntaxError(msg)
 
         # image mode
         if imagetype in (3, 11):
@@ -89,7 +90,8 @@ class TgaImageFile(ImageFile.ImageFile):
             if depth == 32:
                 self.mode = "RGBA"
         else:
-            raise SyntaxError("unknown TGA mode")
+            msg = "unknown TGA mode"
+            raise SyntaxError(msg)
 
         # orientation
         orientation = flags & 0x30
@@ -99,7 +101,8 @@ class TgaImageFile(ImageFile.ImageFile):
         elif orientation in [0, 0x10]:
             orientation = -1
         else:
-            raise SyntaxError("unknown TGA orientation")
+            msg = "unknown TGA orientation"
+            raise SyntaxError(msg)
 
         self.info["orientation"] = orientation
 
@@ -152,7 +155,7 @@ class TgaImageFile(ImageFile.ImageFile):
 
     def load_end(self):
         if self._flip_horizontally:
-            self.im = self.im.transpose(Image.FLIP_LEFT_RIGHT)
+            self.im = self.im.transpose(Image.Transpose.FLIP_LEFT_RIGHT)
 
 
 #
@@ -175,7 +178,8 @@ def _save(im, fp, filename):
     try:
         rawmode, bits, colormaptype, imagetype = SAVE[im.mode]
     except KeyError as e:
-        raise OSError(f"cannot write mode {im.mode} as TGA") from e
+        msg = f"cannot write mode {im.mode} as TGA"
+        raise OSError(msg) from e
 
     if "rle" in im.encoderinfo:
         rle = im.encoderinfo["rle"]
@@ -193,9 +197,10 @@ def _save(im, fp, filename):
         warnings.warn("id_section has been trimmed to 255 characters")
 
     if colormaptype:
-        colormapfirst, colormaplength, colormapentry = 0, 256, 24
+        palette = im.im.getpalette("RGB", "BGR")
+        colormaplength, colormapentry = len(palette) // 3, 24
     else:
-        colormapfirst, colormaplength, colormapentry = 0, 0, 0
+        colormaplength, colormapentry = 0, 0
 
     if im.mode in ("LA", "RGBA"):
         flags = 8
@@ -210,7 +215,7 @@ def _save(im, fp, filename):
         o8(id_len)
         + o8(colormaptype)
         + o8(imagetype)
-        + o16(colormapfirst)
+        + o16(0)  # colormapfirst
         + o16(colormaplength)
         + o8(colormapentry)
         + o16(0)
@@ -225,7 +230,7 @@ def _save(im, fp, filename):
         fp.write(id_section)
 
     if colormaptype:
-        fp.write(im.im.getpalette("RGB", "BGR"))
+        fp.write(palette)
 
     if rle:
         ImageFile._save(
