@@ -376,11 +376,8 @@ PyImaging_BcnDecoderNew(PyObject *self, PyObject *args) {
             actual = "L";
             break;
         case 5: /* BC5: 2-channel 8-bit via 2 BC3 alpha blocks */
-            actual = "RGB";
-            break;
         case 6: /* BC6: 3-channel 16-bit float */
-            /* TODO: support 4-channel floating point images */
-            actual = "RGBAF";
+            actual = "RGB";
             break;
         default:
             PyErr_SetString(PyExc_ValueError, "block compression type unknown");
@@ -433,7 +430,8 @@ PyImaging_GifDecoderNew(PyObject *self, PyObject *args) {
     char *mode;
     int bits = 8;
     int interlace = 0;
-    if (!PyArg_ParseTuple(args, "s|ii", &mode, &bits, &interlace)) {
+    int transparency = -1;
+    if (!PyArg_ParseTuple(args, "s|iii", &mode, &bits, &interlace, &transparency)) {
         return NULL;
     }
 
@@ -451,6 +449,7 @@ PyImaging_GifDecoderNew(PyObject *self, PyObject *args) {
 
     ((GIFDECODERSTATE *)decoder->state.context)->bits = bits;
     ((GIFDECODERSTATE *)decoder->state.context)->interlace = interlace;
+    ((GIFDECODERSTATE *)decoder->state.context)->transparency = transparency;
 
     return (PyObject *)decoder;
 }
